@@ -28,7 +28,7 @@
  */
 
 /*  #ifndef lint */
-/*  static char rcsid[] = "$Header: /user6/ouster/wish/RCS/tkMain.c,v 1.99 93/11/11 09:35:24 ouster Exp $ SPRITE (Berkeley)"; */
+/*  static char rcsid[] = "$Header$ SPRITE (Berkeley)"; */
 /*  #endif */
 
 #include <string.h>
@@ -42,6 +42,11 @@
 #if (TK_MINOR_VERSION==0 && TK_MAJOR_VERSION==4)
 #define TkCreateMainWindow Tk_CreateMainWindow
 #endif
+#if TCL_MAJOR_VERSION >= 8
+#define INTERP_RESULT(interp) Tcl_GetStringResult(interp)
+#else
+#define INTERP_RESULT(interp) (interp)->result
+#endif
 
 
 /*-------------------------------------------------------------------*/
@@ -50,6 +55,8 @@
 #include <signal.h>
 #include <errno.h>
 
+int writable_malloc=0; /*FIXME, don't wrap fopen here, exclude notcomp.h or equivalent */
+
 #include "guis.h"
 struct connection_state *dsfd;
 /*-------------------------------------------------------------------*/
@@ -71,7 +78,7 @@ extern char *		strrchr _ANSI_ARGS_((CONST char *string, int c));
  * Global variables used by the main program:
  */
 
-static Tk_Window mainWindow;	/* The main window for the application.  If
+/* static Tk_Window mainWindow;    The main window for the application.  If
 				 * NULL then the application no longer
 				 * exists. */
 static Tcl_Interp *interp;	/* Interpreter for this application. */
@@ -121,7 +128,7 @@ static void
 TCL_signal_error(x)
      char *x;
 {char buf[300] ;
- sprintf("error %s",x);
+  sprintf(buf,"error %s",x);
  Tcl_Eval(interp,buf);
  dfprintf(stderr,x);
 }
@@ -190,7 +197,8 @@ TkX_Wish (argc, argv)
     int argc;				/* Number of arguments. */
     char **argv;			/* Array of argument strings. */
 {
-    char *args, *p, *msg;
+    char *args, *p;
+    const char *msg;
     char buf[20];
     int code;
 
@@ -203,9 +211,9 @@ TkX_Wish (argc, argv)
      * Parse command-line arguments.
      */
 
-    if (Tk_ParseArgv(interp, (Tk_Window) NULL, &argc, argv, argTable, 0)
+    if (Tk_ParseArgv(interp, (Tk_Window) NULL, &argc, (const char **)argv, argTable, 0)
 	    != TCL_OK) {
-	fprintf(stderr, "%s\n", interp->result);
+	fprintf(stderr, "%s\n", INTERP_RESULT(interp));
 	exit(1);
     }
     if (name == NULL) {
@@ -236,18 +244,18 @@ TkX_Wish (argc, argv)
      * Initialize the Tk application.
      */
 
-    mainWindow = TkCreateMainWindow(interp, display, name/*  , "Tk" */); 
-    if (mainWindow == NULL) {
-	fprintf(stderr, "%s\n", interp->result);
-	exit(1);
-    }
-#ifndef __MINGW32__    
-    if (synchronize) {
-	XSynchronize(Tk_Display(mainWindow), True);
-    }
-#endif    
-    Tk_GeometryRequest(mainWindow, 200, 200);
-    Tk_UnmapWindow(mainWindow);
+/*     mainWindow = TkCreateMainWindow(interp, display, name/\*  , "Tk" *\/);  */
+/*     if (mainWindow == NULL) { */
+/* 	fprintf(stderr, "%s\n", INTERP_RESULT(interp)); */
+/* 	exit(1); */
+/*     } */
+/* #ifndef __MINGW32__     */
+/*     if (synchronize) { */
+/* 	XSynchronize(Tk_Display(mainWindow), True); */
+/*     } */
+/* #endif     */
+/*     Tk_GeometryRequest(mainWindow, 200, 200); */
+/*     Tk_UnmapWindow(mainWindow); */
 
     /*
      * Make command-line arguments available in the Tcl variables "argc"
@@ -255,7 +263,7 @@ TkX_Wish (argc, argv)
      * specified on the command line.
      */
 
-    args = Tcl_Merge(argc-1, argv+1);
+    args = Tcl_Merge(argc-1, (const char **)argv+1);
     Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
     ckfree(args);
     sprintf(buf, "%d", argc-1);
@@ -279,17 +287,17 @@ TkX_Wish (argc, argv)
      * interpreter.
      */
 
-#ifdef SQUARE_DEMO
-    Tcl_CreateCommand(interp, "square", SquareCmd, (ClientData) mainWindow,
-	    (void (*)()) NULL);
-#endif
+/* #ifdef SQUARE_DEMO */
+/*     Tcl_CreateCommand(interp, "square", SquareCmd, (ClientData) mainWindow, */
+/* 	    (void (*)()) NULL); */
+/* #endif */
 
     /*
      * Invoke application-specific initialization.
      */
 
     if (Tcl_AppInit(interp) != TCL_OK) {
-	fprintf(stderr, "Tcl_AppInit failed: %s\n", interp->result);
+	fprintf(stderr, "Tcl_AppInit failed: %s\n", INTERP_RESULT(interp));
     }
 
     /*
@@ -299,7 +307,7 @@ TkX_Wish (argc, argv)
     if (geometry != NULL) {
 	code = Tcl_VarEval(interp, "wm geometry . ", geometry, (char *) NULL);
 	if (code != TCL_OK) {
-	    fprintf(stderr, "%s\n", interp->result);
+	    fprintf(stderr, "%s\n", INTERP_RESULT(interp));
 	}
     }
 
@@ -329,13 +337,13 @@ TkX_Wish (argc, argv)
     
 	    fullName = Tcl_TildeSubst(interp, tcl_RcFileName, &buffer);
 	    if (fullName == NULL) {
-		fprintf(stderr, "%s\n", interp->result);
+		fprintf(stderr, "%s\n", INTERP_RESULT(interp));
 	    } else {
 		f = fopen(fullName, "r");
 		if (f != NULL) {
 		    code = Tcl_EvalFile(interp, fullName);
 		    if (code != TCL_OK) {
-			fprintf(stderr, "%s\n", interp->result);
+			fprintf(stderr, "%s\n", INTERP_RESULT(interp));
 		    }
 		    fclose(f);
 		}
@@ -370,7 +378,7 @@ TkX_Wish (argc, argv)
 error:
     msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
     if (msg == NULL) {
-	msg = interp->result;
+	msg = INTERP_RESULT(interp);
     }
     dfprintf(stderr, "%s\n", msg);
     Tcl_Eval(interp, errorExitCmd);
@@ -396,7 +404,7 @@ tell_lisp_var_changed(
 {
 
   if (being_set_by_lisp == 0)
-    { char *val = Tcl_GetVar2(interp,name1,name2, TCL_GLOBAL_ONLY);
+    { const char *val = Tcl_GetVar2(interp,name1,name2, TCL_GLOBAL_ONLY);
       char buf[3];
       STORE_3BYTES(buf,(long) clientData);
       if(sock_write_str2(dsfd,   m_set_lisp_loc, buf, 3 ,
@@ -494,14 +502,14 @@ StdinProc(clientData, mask)
 	  if (msg->type == m_tcl_command_wait_response
 	      || code)
 	    {
-	      unsigned char buf[4];
-	      unsigned char *p = buf;
+	      char buf[4];
+	      char *p = buf, *string;
 	      /*header */
 	      *p++ = (code ? '1' : '0');
 	      bcopy(msg->msg_id,p,3);
 	      /* end header */
-	      if(sock_write_str2(dsfd, m_reply, buf, 4 ,
-				 interp->result, strlen(interp->result))
+	      string = (char *)INTERP_RESULT(interp);
+	      if(sock_write_str2(dsfd, m_reply, buf, 4, string, strlen(string))
 		 < 0)
 		{		/* what do we want to do if the write failed */}
 	      
@@ -685,7 +693,7 @@ guiBindCallback(char *szNameCmdProc, char *szTclObject, char *szModifier,char* a
   code = Tcl_Eval(interp, szCmd);
   if (code != TCL_OK)
     {
-      dfprintf(stderr, "TCL Error int bind : %s\n", interp->result);
+      dfprintf(stderr, "TCL Error int bind : %s\n", INTERP_RESULT(interp));
 
     }
   return code;
