@@ -15,6 +15,6 @@
 #define RELOC_H "elf64_alpha_reloc.h"
 #define SPECIAL_RELOC_H "elf64_alpha_reloc_special.h"
 #define PAL_imb		134
-#define imb() \
-__asm__ __volatile__ ("call_pal %0 #imb" : : "i" (PAL_imb) : "memory")
+#define imb() __asm__ __volatile__ ("call_pal %0 #imb" : : "i" (PAL_imb) : "memory")
 #define CLEAR_CACHE imb()
+
