@@ -97,23 +97,41 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "../h/mp.h"
 #endif
 
+#include <setjmp.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <math.h>
+#include <complex.h>
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
+#ifdef HAVE_READLINE
+#include <readline/readline.h>
+#endif
+
+#include "../h/sfun_argd.h"
 #include "../h/compbas.h"
 #include "../h/enum.h"
-#include "../h/bits.h"
+#include "../h/pageinfo.h"
+#include "../h/lu.h"
+#include "../h/globals.h"
+#include "../h/fixnum.h"
+#include "../h/type.h"
 #include "../h/object.h"
 #include "../h/vs.h"
 #include "../h/bds.h"
 #include "../h/frame.h"
-#include "../h/stacks.h"
 #include "../h/lex.h"
 #include "../h/eval.h"
 
 #include "../h/protoize.h"
+#include "../h/compprotos.h"
 #include "../h/notcomp.h"
 #include "../h/funlink.h"
 #include "../h/att_ext.h"
+#ifndef INICOMP
 #include "../h/new_decl.h"
-#include "compbas2.h"
+#endif
 #include "compat.h"
 
 #include "../h/rgbc.h"
