@@ -79,8 +79,6 @@ do {static struct sigaction action; \
 
 #undef HAVE_SIGVEC
 #define HAVE_SIGACTION
-/* make this a noop */
-/* #define SETUP_SIG_STACK */
 #ifndef HAVE_SV_ONSTACK
 #define SV_ONSTACK 0
 #endif
@@ -105,13 +103,10 @@ do {static struct sigaction action; \
 #undef LISTEN_FOR_INPUT
 #define LISTEN_FOR_INPUT(fp) \
 do { int c = 0; \
-  if(((fp)->_IO_read_ptr >= (fp)->_IO_read_end) \
-     && (ioctl((fp)->_fileno, FIONREAD, &c),c<=0)) \
+  if((((FILE *)fp)->_IO_read_ptr >= ((FILE *)fp)->_IO_read_end)	\
+     && (ioctl(((FILE *)fp)->_fileno, FIONREAD, &c),c<=0))	\
      return 0;} while (0)
 
-/* we dont need to worry about zeroing fp->_base , to prevent  */
-#define FCLOSE_SETBUF_OK 
-
 /* #define DATA_BEGIN((TXTRELOC+header.a_text+(SEGSIZ-1)) & ~(SEGSIZ-1)); */
 #define DATA_BEGIN (char *)(char *)N_DATADDR(header);
 
@@ -123,11 +118,6 @@ do { int c = 0; \
 #define GETPATHNAME
 #define PATHNAME_CACHE	10
 
-
-#define INSTALL_SEGMENTATION_CATCHER \
-  	 (void) gcl_signal(SIGSEGV,segmentation_catcher)
-
-
 /* get the fileno of a FILE* */
 #define FILENO(x) fileno(x)
 
@@ -144,8 +134,6 @@ do { int c = 0; \
   setbuf(stdin,0); \
    setbuf(stdout,0);
 
-#define INIT_CORE_END terminal_io->sm.sm_object0->sm.sm_fp = stdin;terminal_io->sm.sm_object1->sm.sm_fp = stdout;
-
 #include <limits.h>
 #include <sys/stat.h>
 #define GET_FULL_PATH_SELF(a_) do {\
@@ -162,3 +150,46 @@ do { int c = 0; \
    (a_)=q;\
  }\
 } while(0)
+
+
+#define UC(a_) ((ucontext_t *)a_)
+#define SF(a_) ((siginfo_t *)a_)
+
+#if defined(__linux__) && (defined(__x86_64__) || defined(__i386__))
+
+/* #define FPE_CODE(i_) make_fixnum((fixnum)SF(i_)->si_code) */
+#ifdef __i386__
+#define FPE_CODE(i_,v_) make_fixnum(FFN(fSfpe_code)(UC(v_)->uc_mcontext.fpregs->sw,((struct _fpstate *)UC(v_)->uc_mcontext.fpregs)->mxcsr))
+#define FPE_ADDR(i_,v_) make_fixnum((UC(v_)->uc_mcontext.fpregs->tag!=-1) ? UC(v_)->uc_mcontext.fpregs->ipoff : (fixnum)SF(i_)->si_addr)
+#else
+#define FPE_CODE(i_,v_) make_fixnum(FFN(fSfpe_code)(UC(v_)->uc_mcontext.fpregs->swd,((struct _fpstate *)UC(v_)->uc_mcontext.fpregs)->mxcsr))
+#define FPE_ADDR(i_,v_) make_fixnum(UC(v_)->uc_mcontext.fpregs->fop ? UC(v_)->uc_mcontext.fpregs->rip : (fixnum)SF(i_)->si_addr)
+#endif
+#define FPE_CTXT(v_) list(3,make_fixnum((fixnum)&UC(v_)->uc_mcontext.gregs),	\
+			    make_fixnum((fixnum)&UC(v_)->uc_mcontext.fpregs->_st), \
+			    make_fixnum((fixnum)&((struct _fpstate *)UC(v_)->uc_mcontext.fpregs)->_xmm))
+
+#define MC(b_) v.uc_mcontext.b_
+#define REG_LIST(a_) MMcons(make_fixnum(sizeof(a_)),make_fixnum(sizeof(*a_)))
+#define MCF(b_) (((struct _fpstate *)MC(fpregs))->b_)
+
+#ifdef __x86_64__
+#define FPE_RLST "R8 R9 R10 R11 R12 R13 R14 R15 RDI RSI RBP RBX RDX RAX RCX RSP RIP EFL CSGSFS ERR TRAPNO OLDMASK CR2"
+#elif defined(__i386__)
+#define FPE_RLST "GS FS ES DS EDI ESI EBP ESP EBX EDX ECX EAX TRAPNO ERR EIP CS EFL UESP SS"
+#else
+#error Missing reg list
+#endif
+
+#define FPE_INIT ({ucontext_t v;list(3,MMcons(make_simple_string(({const char *s=FPE_RLST;s;})),REG_LIST(MC(gregs))),\
+				     REG_LIST(MCF(_st)),REG_LIST(MCF(_xmm)));})
+
+#else
+
+#define FPE_CODE(i_,v_) make_fixnum((fixnum)SF(i_)->si_code)
+#define FPE_ADDR(i_,v_) make_fixnum((fixnum)SF(i_)->si_addr)
+#define FPE_CTXT(v_) Cnil
+
+#define FPE_INIT Cnil
+
+#endif
