@@ -1,15 +1,16 @@
-/* alloc.c:89:OF */ extern char *alloc_page (long n); /* (n) int n; */
-/* alloc.c:149:OF */ extern void add_page_to_freelist (char *p, struct typemanager *tm); /* (p, tm) char *p; struct typemanager *tm; */
+/* alloc.c:89:OF */ extern void *alloc_page (long n); /* (n) int n; */
+/* alloc.c:149:OF */ inline void add_page_to_freelist (char *p, struct typemanager *tm); /* (p, tm) char *p; struct typemanager *tm; */
 /* alloc.c:196:OF */ extern object type_name (int t); /* (t) int t; */
-/* alloc.c:213:OF */ extern object alloc_object (enum type t); /* (t) enum type t; */
-/* alloc.c:296:OF */ extern object make_cons (object a, object d); /* (a, d) object a; object d; */
+/* alloc.c:213:OF */ inline object alloc_object (enum type t); /* (t) enum type t; */
+/* alloc.c:213:OF */ inline void add_pages(struct typemanager *,fixnum);
+/* alloc.c:296:OF */ extern inline object make_cons (object a, object d); /* (a, d) object a; object d; */
 /* alloc.c:364:OF */ extern object on_stack_cons (object x, object y); /* (x, y) object x; object y; */
 /* alloc.c:376:OF */ extern object fSallocated (object typ); /* (typ) object typ; */
 /* alloc.c:401:OF */ extern object fSreset_number_used (object typ); /* (typ) object typ; */
 /* alloc.c:480:OF */ extern void insert_contblock (char *p, int s); /* (p, s) char *p; int s; */
 /* alloc.c:480:OF */ extern void insert_maybe_sgc_contblock (char *p, int s); /* (p, s) char *p; int s; */
 /* alloc.c:611:OF */ extern void set_maxpage (void); /* () */
-/* alloc.c:635:OF */ extern void gcl_init_alloc (void); /* () */
+/* alloc.c:635:OF */ extern void gcl_init_alloc (void *); /* () */
 /* alloc.c:737:OF */ extern object fSstaticp (object x); /* (x) object x; */
 /* alloc.c:822:OF */ extern object fSallocate_sgc (object type,fixnum min,fixnum max,fixnum free_percent); /* (type, min, max, free_percent) object type; int min; int max; int free_percent; */
 /* alloc.c:846:OF */ extern object fSallocate_growth (object type,fixnum min,fixnum max,fixnum percent,fixnum percent_free); /* (type, min, max, percent, percent_free) object type; int min; int max; int percent; int percent_free; */
@@ -32,7 +33,7 @@
 /* array.c::OF */ extern object fSmake_vector1_1 (fixnum n,fixnum elt_type,object staticp); 
 /* array.c:738:OF */ extern void adjust_displaced (object x, long diff); /* (x, diff) object x; int diff; */
 /* array.c:790:OF */ extern void gset (void *p1, void *val, int n, int typ); /* (p1, val, n, typ) char *p1; char *val; int n; int typ; */
-/* array.c:831:OF */ extern object fScopy_array_portion (object x, object y,fixnum i1,fixnum i2, object n1); /* (x, y, i1, i2, n1) object x; object y; int i1; int i2; int n1; */
+/* array.c:831:OF */ extern object fScopy_array_portion (object x, object y,object i1,object i2, object n1); /* (x, y, i1, i2, n1) object x; object y; int i1; int i2; int n1; */
 /* array.c:879:OF */ extern void array_allocself (object x, int staticp, object dflt); /* (x, staticp, dflt) object x; int staticp; object dflt; */
 /* array.c:920:OF */ extern void siLfill_pointer_set (void); /* () */
 /* array.c:923:OF */ extern object fSfill_pointer_set (object x,fixnum i); /* (x, i) object x; int i; */
@@ -51,7 +52,6 @@
 /* array.c:1023:OF */ extern object fLarray_dimension (object x,fixnum i); /* (x, i) object x; int i; */
 /* array.c:1090:OF */ extern void siLreplace_array (void); /* () */
 /* array.c:1093:OF */ extern object fSreplace_array (object old, object new); /* (old, new) object old; object new; */
-/* array.c:1132:OF */ extern object fLarray_total_size (object x); /* (x) object x; */
 /* array.c:1140:OF */ extern object fSaset_by_cursor (object array, object val, object cursor); /* (array, val, cursor) object array; object val; object cursor; */
 /* array.c:1160:OF */ extern void gcl_init_array_function (void); /* () */
 /* assignment.c:62:OF */ extern void setq (object sym, object val); /* (sym, val) object sym; object val; */
@@ -81,7 +81,7 @@
 /* gmp_big.c:324:OF */ extern double big_to_double (object x); /* (x) object x; */
 /* gmp_big.c:454:OF */ extern object maybe_replace_big (object x); /* (x) object x; */
 /* gmp_big.c:472:OF */ extern object bignum2 (unsigned int h, unsigned int l); /* (h, l) unsigned int h; unsigned int l; */
-/* gmp_big.c:482:OF */ extern void integer_quotient_remainder_1 (object x, object y, object *qp, object *rp); /* (x, y, qp, rp) object x; object y; object *qp; object *rp; */
+/* gmp_big.c:482:OF */ extern void integer_quotient_remainder_1 (object x, object y, object *qp, object *rp,fixnum z); /* (x, y, qp, rp) object x; object y; object *qp; object *rp; */
 /* gmp_big.c:502:OF */ extern object coerce_big_to_string (object x, int printbase); /* (x, printbase) object x; int printbase; */
 /* gmp_big.c:521:OF */ extern void gcl_init_big (void); /* () */
 /* big.c:72:OF */ extern int big_sign (object x); /* (x) object x; */
@@ -141,11 +141,12 @@ struct key {short n,allow_other_keys;
 /* cmpaux.c:106:OF */ /* extern int ifloor (int x, int y); */ /* (x, y) int x; int y; */
 /* cmpaux.c:124:OF */ /* extern int imod (int x, int y); */ /* (x, y) int x; int y; */
 /* cmpaux.c:185:OF */ extern int object_to_int (object x); /* (x) object x; */
+/* cmpaux.c:185:OF */ extern fixnum object_to_fixnum (object x); /* (x) object x; */
 /* cmpaux.c:263:OF */ extern char *object_to_string (object x); /* (x) object x; */
 typedef int (*FUNC)();
 /* cmpaux.c:294:OF */ extern void call_init (int init_address, object memory, object fasl_vec, FUNC fptr); /* (init_address, memory, fasl_vec, fptr) int init_address; object memory; object fasl_vec; FUNC fptr; */
 /* cmpaux.c:339:OF */ extern void do_init (object *statVV); /* (statVV) object *statVV; */
-/* cmpaux.c:416:OF */ extern void gcl_init_or_load1 (void (*fn) (void), char *file); /* (fn, file) int (*fn)(); char *file; */
+/* cmpaux.c:416:OF */ extern void gcl_init_or_load1 (void (*fn) (void), const char *file); /* (fn, file) int (*fn)(); char *file; */
 /* conditional.c:200:OF */ extern void gcl_init_conditional (void); /* () */
 /* error.c:38:OF */ extern void terminal_interrupt (int correctable); /* (correctable) int correctable; */
 /* error.c:147:OF */ extern void Lerror (void); /* () */
@@ -191,7 +192,6 @@ typedef int (*FUNC)();
 /* error.c:641:OF */ extern void not_a_symbol (object obj); /* (obj) object obj; */
 /* error.c:647:OF */ extern int not_a_variable (object obj); /* (obj) object obj; */
 /* error.c:653:OF */ extern void illegal_index (object x, object i); /* (x, i) object x; object i; */
-/* error.c:660:OF */ extern void check_socket (object x); /* (x) object x; */
 /* error.c:670:OF */ extern void check_stream (object strm); /* (strm) object strm; */
 /* error.c:697:OF */ extern void check_arg_range (int n, int m); /* (n, m) int n; int m; */
 /* error.c:727:OF */ extern void gcl_init_error (void); /* () */
@@ -225,26 +225,25 @@ typedef void (*funcvoid)(void);
 /* format.c::OF */ extern object fLformat_1(object strm, object control,object x);
 /* format.c:2084:OF */ extern void Lformat (void); /* () */
 /* format.c:2171:OF */ extern void gcl_init_format (void); /* () */
-/* frame.c:32:OF */ extern void unwind (frame_ptr fr, object tag); /* (fr, tag) frame_ptr fr; object tag; */
+/* frame.c:32:OF */ extern void unwind (frame_ptr fr, object tag) NO_RETURN; /* (fr, tag) frame_ptr fr; object tag; */
 /* frame.c:58:OF */ extern frame_ptr frs_sch (object frame_id); /* (frame_id) object frame_id; */
 /* frame.c:69:OF */ extern frame_ptr frs_sch_catch (object frame_id); /* (frame_id) object frame_id; */
 /* funlink.c:19:OF */ extern void call_or_link (object sym, void **link); /* (sym, link) object sym; void **link; */
 /* funlink.c:41:OF */ extern void call_or_link_closure (object sym, void **link, void **ptr); /* (sym, link, ptr) object sym; void **link; object *ptr; */
-/* funlink.c:230:OF */ extern object c_apply_n (object (*fn)(), int n, object *x); /* (fn, n, x) long int (*fn)(); int n; object *x; */
 /* funlink.c:696:OF */ extern object call_proc0 (object sym, void *link); /* (sym, link) object sym; void *link; */
 /* funlink.c:784:OF */ extern int clear_stack (object *beg, object *limit); /* (beg, limit) object *beg; object *limit; */
 /* funlink.c:821:OF */ extern void gcl_init_links (void); /* () */
 /* gbc.c:151:OF */ extern void enter_mark_origin (object *p); /* (p) object *p; */
 /* gbc.c:938:OF */ extern void GBC (enum type t); /* (t) enum type t; */
 /* gbc.c:1326:OF */ extern object fLgbc (object x0); /* (x0) object x0; */
-/* sgbc.c:924:OF */ extern int sgc_count_type (int t); /* (t) int t; */
+/* sgbc.c:924:OF */ extern fixnum sgc_count_type (int t); /* (t) int t; */
 /* sgbc.c:938:OF */ extern int sgc_start (void); /* () */
 /* sgbc.c:1068:OF */ extern int sgc_quit (void); /* () */
 /* sgbc.c:1131:OF */ extern void make_writable (unsigned long beg, unsigned long i); /* (beg, i) int beg; int i; */
 #ifndef __MINGW32__
 /* #include <signal.h> */
 #endif
-/* sgbc.c:1246:OF */ extern void memory_protect (int on); /* (on) int on; */
+/* sgbc.c:1246:OF */ extern int memory_protect (int on); /* (on) int on; */
 /* sgbc.c:1306:OF */ extern void perm_writable (char *p, long n); /* (p, n) char *p; int n; */
 /* sgbc.c:1321:OF */ extern void system_error (void); /* () */
 /* gbc.c:1357:OF */ extern void gcl_init_GBC (void); /* () */
@@ -308,8 +307,12 @@ typedef void (*funcvoid)(void);
 /* main.c:807:OF */ extern object fLlisp_implementation_version (void); /* () */
 /* makefun.c:10:OF */ extern object MakeAfun (object (*addr)(object,object), unsigned int argd, object data); /* (addr, argd, data) int (*addr)(); unsigned int argd; object data; */
 /* makefun.c:113:OF */ extern object fSset_key_struct (object key_struct_ind); /* (key_struct_ind) object key_struct_ind; */
-/* makefun.c:122:OF */ extern void SI_makefun (char *strg, object (*fn) (/* ??? */), unsigned int argd); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
-/* makefun.c:131:OF */ extern void LISP_makefun (char *strg, object (*fn) (/* ??? */), unsigned int argd); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
+/* /\* makefun.c:122:OF *\/ extern void SI_makefun (char *strg, object (*fn) (/\* ??? *\/), unsigned int argd); /\* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; *\/ */
+/* /\* makefun.c:131:OF *\/ extern void LISP_makefun (char *strg, object (*fn) (/\* ??? *\/), unsigned int argd); /\* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; *\/ */
+/* makefun.c:122:OF */ extern void SI_makefun (char *,void *,unsigned int); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
+/* makefun.c:131:OF */ extern void LISP_makefun (char *,void *,unsigned int); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
+/* makefun.c:122:OF */ extern void SI_makefunm (char *,void *,unsigned int); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
+/* makefun.c:131:OF */ extern void LISP_makefunm (char *,void *,unsigned int); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
 /* makefun.c:167:OF */ extern object fSinvoke (object x); /* (x) object x; */
 /* mapfun.c:324:OF */ extern void gcl_init_mapfun (void); /* () */
 /* multival.c:32:OF */ extern void Lvalues (void); /* () */
@@ -337,7 +340,8 @@ typedef void (*funcvoid)(void);
 /* num_arith.c:478:OF */ extern object number_negate (object x); /* (x) object x; */
 /* num_arith.c:520:OF */ extern object number_times (object x, object y); /* (x, y) object x; object y; */
 /* num_arith.c:670:OF */ extern object number_divide (object x, object y); /* (x, y) object x; object y; */
-/* num_arith.c:818:OF */ extern object integer_divide1 (object x, object y); /* (x, y) object x; object y; */
+/* num_arith.c:818:OF */ extern object integer_divide1 (object x, object y,fixnum z); /* (x, y) object x; object y; */
+/* num_arith.c:818:OF */ extern object integer_divide2 (object x, object y,fixnum z,object *r); /* (x, y) object x; object y; */
 /* num_arith.c:828:OF */ extern object get_gcd (object x, object y); /* (x, y) object x; object y; */
 /* num_arith.c:873:OF */ extern void Lplus (void); /* () */
 /* num_arith.c:889:OF */ extern void Lminus (void); /* () */
@@ -365,7 +369,7 @@ typedef void (*funcvoid)(void);
 /* num_comp.c:272:OF */ extern void Lmonotonically_nonincreasing (void); /* () */
 /* num_comp.c:292:OF */ extern void Lmin (void); /* () */
 /* num_comp.c:309:OF */ extern void gcl_init_num_comp (void); /* () */
-/* num_log.c:224:OF */ extern object shift_integer (object x, int w); /* (x, w) object x; int w; */
+/* num_log.c:224:OF */ extern object integer_fix_shift (object x, fixnum w); /* (x, w) object x; int w; */
 /* num_log.c:258:OF */ extern void Llogior (void); /* () */
 /* num_log.c:279:OF */ extern void Llogxor (void); /* () */
 /* num_log.c:299:OF */ extern void Llogand (void); /* () */
@@ -434,11 +438,14 @@ typedef void (*funcvoid)(void);
 /* predicate.c:346:OF */ extern object fLcompiled_function_p (object x0); /* (x0) object x0; */
 /* predicate.c:367:OF */ extern object fLcommonp (object x0); /* (x0) object x0; */
 /* predicate.c:379:OF */ extern object fLeq (object x0, object x1); /* (x0, x1) object x0; object x1; */
-/* predicate.c:393:OF */ extern int eql (object x, object y); /* (x, y) object x; object y; */
+/* predicate.c:393:OF */ extern bool eql1 (object x, object y); /* (x, y) object x; object y; */
+/* predicate.c:393:OF */ extern bool oeql (object x, object y); /* (x, y) object x; object y; */
 /* predicate.c:455:OF */ extern object fLeql (object x0, object x1); /* (x0, x1) object x0; object x1; */
-/* predicate.c:469:OF */ extern int equal (register object x, register object y); /* (x, y) register object x; register object y; */
+/* predicate.c:469:OF */ extern bool equal1 (register object x, register object y); /* (x, y) register object x; register object y; */
+/* predicate.c:469:OF */ extern bool oequal (register object x, register object y); /* (x, y) register object x; register object y; */
 /* predicate.c:543:OF */ extern object fLequal (object x0, object x1); /* (x0, x1) object x0; object x1; */
-/* predicate.c:557:OF */ extern bool equalp (object x, object y); /* (x, y) object x; object y; */
+/* predicate.c:557:OF */ extern bool equalp1 (object x, object y); /* (x, y) object x; object y; */
+/* predicate.c:557:OF */ extern bool oequalp (object x, object y); /* (x, y) object x; object y; */
 /* predicate.c:681:OF */ extern object fLequalp (object x0, object x1); /* (x0, x1) object x0; object x1; */
 /* predicate.c:750:OF */ extern bool contains_sharp_comma (object x); /* (x) object x; */
 /* predicate.c:797:OF */ extern object fScontains_sharp_comma (object x0); /* (x0) object x0; */
@@ -464,8 +471,8 @@ typedef void (*funcvoid)(void);
 #include <unistd.h>
 /* sbrk.c:9:OF */ /*  extern void * sbrk (int n); */ /* (n) int n; */
 /* strcspn.c:3:OF */ /*  extern size_t strcspn (const char *s1, const char *s2); */ /* (s1, s2) char *s1; char *s2; */
-/* structure.c:59:OF */ extern object structure_ref (object x, object name, int i); /* (x, name, i) object x; object name; int i; */
-/* structure.c:107:OF */ extern object structure_set (object x, object name, int i, object v); /* (x, name, i, v) object x; object name; int i; object v; */
+/* structure.c:59:OF */ extern object structure_ref (object x, object name, fixnum i); /* (x, name, i) object x; object name; int i; */
+/* structure.c:107:OF */ extern object structure_set (object x, object name, fixnum i, object v); /* (x, name, i, v) object x; object name; int i; object v; */
 /* structure.c:164:OF */ extern object structure_to_list (object x); /* (x) object x; */
 /* structure.c:188:OF */ extern void siLmake_structure (void); /* () */
 /* structure.c:281:OF */ extern void siLstructure_set (void); /* () */
@@ -549,6 +556,9 @@ typedef void (*funcvoid)(void);
 
 /*  readline.d */
 extern int readline_on;
+void
+gcl_init_readline(void);
+
 void
 gcl_init_readline_function(void);
 
@@ -651,7 +661,7 @@ int
 pack_hash(object);
 
 void
-load(char *);
+load(const char *);
 
 bool
 member_eq(object,object);
@@ -1280,17 +1290,6 @@ Ldigit_char_p(void);
 void
 Lwrite_byte(void);
 
-#ifdef SPECIAL_RSYM
-void
-read_special_symbols(char *);
-
-int
-node_compare(const void *,const void *);
-#endif
-
-void
-unlink_loaded_files(void);
-
 void
 FEpackage_error(object,const char *s);
 
@@ -1767,6 +1766,9 @@ gcl_isnormal_double(double);
 int 
 gcl_isnormal_float(float);
 
+object
+find_init_name1(char *,unsigned);
+
 #ifdef SGC
 void
 memprotect_test_reset(void);
@@ -1778,18 +1780,146 @@ int bcmp ( const void *s1, const void *s2, size_t n );
 void bcopy ( const void *s1, void *s2, size_t n );
 void bzero(void *b, size_t length);
 int TcpOutputProc ( int fd, char *buf, int toWrite, int *errorCodePtr, int block );
-void recreate_heap1 ( void );
 void gcl_init_shared_memory ( void );
 void fix_filename ( object pathname, char *filename1 );
 void alarm ( int n );
-void *sbrk ( unsigned long increment );
+void *sbrk ( ptrdiff_t increment );
 void sigemptyset( sigset_t *set);
 void sigaddset ( sigset_t *set, int n);
 int sigismember ( sigset_t *set, int n );
 int sigprocmask ( int how, const sigset_t *set, sigset_t *oldset );
 #endif
 
+#if defined (__MINGW32__) || defined (__CYGWIN__)
+void recreate_heap1 ( void );
+#endif
+
 #ifdef GCL_GPROF
 void
 gprof_cleanup(void);
 #endif
+
+int
+msystem(const char *);
+
+void
+assert_error(const char *,unsigned,const char *,const char *);
+
+#ifdef _WIN32
+void
+detect_wine(void);
+
+void
+init_shared_memory(void);
+
+void *
+alloca(size_t);
+
+object
+find_init_string(const char *);
+
+#endif
+
+void *
+get_mmap(FILE *,void **);
+
+int
+un_mmap(void *,void *);
+
+object
+fSuse_fast_links_2(object,object);
+
+MP_INT *
+otoi(object);
+void
+isetq_fix(MP_INT *,int);
+int
+mpz_to_mpz1(MP_INT *,MP_INT *,void *);
+int
+mpz_to_mpz(MP_INT *,MP_INT *);
+int
+obj_to_mpz1(object,MP_INT *,void *);
+int
+obj_to_mpz(object,MP_INT *);
+
+struct htent *gethash(object,object);
+
+int
+update_real_maxpage(void);
+
+inline fixnum
+set_tm_maxpage(struct typemanager *,fixnum);
+
+void
+init_gmp_rnd_state(__gmp_randstate_struct *);
+
+inline void
+set_sgc_bit(struct pageinfo *,void *);
+
+void
+reinit_gmp(void);
+
+object
+mod(object,object);
+
+inline void
+intdivrem(object,object,fixnum,object *,object *);
+
+inline object
+integer_count(object);
+
+inline object
+integer_length(object);
+
+inline bool
+integer_bitp(object,object);
+
+inline object 
+fixnum_times(fixnum,fixnum);
+
+inline object
+log_op2(fixnum,object,object);
+
+inline object
+fixnum_big_shift(fixnum,fixnum);
+
+inline object
+integer_shift(object,object);
+
+object
+number_abs(object);
+
+object
+number_signum(object);
+
+
+object
+number_ldb(object,object);
+object
+number_ldbt(object,object);
+object
+number_dpb(object,object,object);
+object
+number_dpf(object,object,object);
+
+#if defined(DARWIN)
+void init_darwin_zone_compat ();
+#endif
+
+void
+prelink_init(void);
+
+int
+gcl_mprotect(void *,unsigned long,int);
+
+int
+rl_pending_buffered_input_p(FILE *f);
+
+int
+rl_eof_p(FILE *f);
+
+int
+rl_stream_p(FILE *f);
+
+void
+sigint(void);
