@@ -31,6 +31,10 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "include.h"
 #include "page.h"
 
+#ifdef HAVE_MPROTECT
+#include <sys/mman.h>
+#endif
+
 static int
 t_from_type(object);
 
@@ -415,10 +419,8 @@ opt_maxpage(struct typemanager *my_tm) {
   struct typemanager *tm,*tme;
   long mro=0,tro=0;
 
-#ifdef HAVE_SYSCONF_PHYS_PAGES
-  if (page(heap_end)-first_data_page+nrbpage>=phys_pages)
+  if (phys_pages>0 && page(heap_end)-first_data_page+nrbpage>=phys_pages)
     return 0;
-#endif
 
   if (page(core_end)>0.8*real_maxpage)
     return 0;
@@ -573,7 +575,7 @@ too_full_p(struct typemanager *tm) {
 inline void *
 alloc_after_gc(struct typemanager *tm,fixnum n) {
 
-  if (tm->tm_npage+tpage(tm,n)>=tm->tm_maxpage) {
+  if (tm->tm_npage+tpage(tm,n)>=tm->tm_maxpage && GBC_enable) {
 
     switch (jmp_gmp) {
     case 0: /* not in gmp call*/
@@ -782,7 +784,7 @@ inline object on_stack_cons(object x, object y) {
 }
 
 
-DEFUN_NEW("ALLOCATED",object,fSallocated,SI,1,1,NONE,OO,OO,OO,OO,(object typ),"")
+DEFUNM_NEW("ALLOCATED",object,fSallocated,SI,1,1,NONE,OO,OO,OO,OO,(object typ),"")
 { struct typemanager *tm=(&tm_table[t_from_type(typ)]);
   tm = & tm_table[tm->tm_type];
   if (tm->tm_type == t_relocatable)
@@ -988,25 +990,6 @@ init_tm(enum type t, char *name, int elsize, int nelts, int sgc,int distinct) {
    call is too fragile.  20050115 CM*/
 static int gcl_alloc_initialized;
 
-void
-set_maxpage(void) {
-
-  /* This is run in init.  Various initializations including getting
-     maxpage are here */ 
-#ifdef SGC
-
-  massert(getpagesize()<=PAGESIZE);
-
-  if (gcl_alloc_initialized) {
-    extern long maxpage;
-    maxpage=page(heap_end);
-    memory_protect(sgc_enabled ? 1 : 0);
-  }
-#endif
-  
-  update_real_maxpage();
-
-}
 
 #ifdef GCL_GPROF
 static unsigned long textstart,textend,textpage;
@@ -1025,34 +1008,96 @@ static void init_textpage() {
 }
 #endif
 
-void
-gcl_init_alloc(void) {
+object malloc_list=Cnil;
 
+#include <signal.h>
+
+void
+gcl_init_alloc(void *cs_start) {
+
+  fixnum cssize=(1L<<23);
+
+  prelink_init();
+  
+#ifdef RECREATE_HEAP
+  if (!raw_image) RECREATE_HEAP;
+#endif
+		    
+#if defined(DARWIN)
+  init_darwin_zone_compat ();
+#endif
+  
 #ifdef GCL_GPROF
   init_textpage();
 #endif
   
-  if (gcl_alloc_initialized) return;
+#if defined(BSD) && defined(RLIMIT_STACK)
+  {
+    struct rlimit rl;
   
+  /* Maybe the soft limit for data segment size is lower than the
+   * hard limit.  In that case, we want as much as possible.
+   */
+    massert(!getrlimit(RLIMIT_DATA, &rl));
+    if (rl.rlim_cur != RLIM_INFINITY &&	(rl.rlim_max == RLIM_INFINITY || rl.rlim_max > rl.rlim_cur)) {
+      rl.rlim_cur = rl.rlim_max;
+      massert(!setrlimit(RLIMIT_DATA, &rl));
+    }
+
+    massert(!getrlimit(RLIMIT_STACK, &rl));
+    if (rl.rlim_cur!=RLIM_INFINITY && (rl.rlim_max == RLIM_INFINITY || rl.rlim_max > rl.rlim_cur)) {
+      rl.rlim_cur = rl.rlim_max == RLIM_INFINITY ? rl.rlim_max : rl.rlim_max/64;
+      massert(!setrlimit(RLIMIT_STACK,&rl));
+    }
+    cssize = rl.rlim_cur/sizeof(*cs_org) - sizeof(*cs_org)*CSGETA;
   
-#ifdef BSD
-#ifdef RLIMIT_STACK
- {
+  }
+#endif
+  
+  cs_org = cs_base = cs_start;
+  cs_limit = cs_org + CSTACK_DIRECTION*cssize;
 
-   struct rlimit rl;
-
-   getrlimit(RLIMIT_DATA, &rl);
-   if (rl.rlim_cur != RLIM_INFINITY &&
-       (rl.rlim_max == RLIM_INFINITY ||
-	rl.rlim_max > rl.rlim_cur)) {
-     rl.rlim_cur = rl.rlim_max;
-    setrlimit(RLIMIT_DATA, &rl);
-   }
-
- }
+#ifdef __ia64__
+  {
+    extern void * __libc_ia64_register_backing_store_base;
+    cs_org2=cs_base2=__libc_ia64_register_backing_store_base;
+  }
+#endif
+  
+#ifdef HAVE_SIGALTSTACK
+  {
+    /* make sure the stack is 8 byte aligned */
+    static double estack_buf[32*SIGSTKSZ];
+    static stack_t estack;
+    
+    estack.ss_sp = estack_buf;
+    estack.ss_flags = 0;                                   
+    estack.ss_size = sizeof(estack_buf);                             
+    massert(sigaltstack(&estack, 0)>=0);
+  }
 #endif	
+  
+  install_segmentation_catcher();
+  
+#ifdef HAVE_MPROTECT
+  if (data_start)
+    massert(!gcl_mprotect(data_start,(void *)core_end-data_start,PROT_READ|PROT_WRITE|PROT_EXEC));
 #endif
 
+#ifdef SGC
+
+  massert(getpagesize()<=PAGESIZE);
+  memprotect_test_reset();
+  if (sgc_enabled)
+    if (memory_protect(1))
+      sgc_quit();
+
+#endif
+
+  update_real_maxpage();
+
+  if (gcl_alloc_initialized) return;
+  
 #ifdef INIT_ALLOC  
   INIT_ALLOC;
 #endif  
@@ -1060,8 +1105,6 @@ gcl_init_alloc(void) {
   data_start=heap_end;
   first_data_page=page(data_start);
   
-  set_maxpage();
-  
   holepage=new_holepage;
 
 #ifdef GCL_GPROF
@@ -1076,8 +1119,7 @@ gcl_init_alloc(void) {
      course changeable by allocate-sgc.  CM 20030827 */
 
   init_tm(t_cons, ".CONS", sizeof(struct cons), 65536 ,50,0 );
-  init_tm(t_fixnum, "NFIXNUM",
-	  sizeof(struct fixnum_struct), 8192,20,0);
+  init_tm(t_fixnum, "NFIXNUM",sizeof(struct fixnum_struct), 8192,20,0);
   init_tm(t_structure, "SSTRUCTURE", sizeof(struct structure), 5461,1,0 );
   init_tm(t_cfun, "fCFUN", sizeof(struct cfun), 4096,1,0  );
   init_tm(t_sfun, "gSFUN", sizeof(struct sfun),409,1,0 );
@@ -1086,10 +1128,8 @@ gcl_init_alloc(void) {
   init_tm(t_symbol, "|SYMBOL", sizeof(struct symbol), 3640,1,0 );
   init_tm(t_bignum, "BBIGNUM", sizeof(struct bignum), 2730,1,0 );
   init_tm(t_ratio, "RRATIONAL", sizeof(struct ratio), 170,1,0 );
-  init_tm(t_shortfloat, "FSHORT-FLOAT",
-	  sizeof(struct shortfloat_struct), 256 ,1,0);
-  init_tm(t_longfloat, "LLONG-FLOAT",
-	  sizeof(struct longfloat_struct), 170 ,1,0);
+  init_tm(t_shortfloat, "FSHORT-FLOAT",sizeof(struct shortfloat_struct), 256 ,1,0);
+  init_tm(t_longfloat, "LLONG-FLOAT",sizeof(struct longfloat_struct), 170 ,1,0);
   init_tm(t_complex, "CCOMPLEX", sizeof(struct ocomplex), 170 ,1,0);
   init_tm(t_character,"#CHARACTER",sizeof(struct character), 256 ,1,0);
   init_tm(t_package, ":PACKAGE", sizeof(struct package), 2*PAGESIZE / sizeof(struct package),1,0);
@@ -1113,31 +1153,15 @@ gcl_init_alloc(void) {
   
   ncb = 0;
   ncbpage = 0;
-  set_tm_maxpage(tm_table+t_contiguous,19);
+  set_tm_maxpage(tm_table+t_contiguous,1);
 #ifdef GCL_GPROF
   if (maxcbpage<textpage)
     set_tm_maxpage(tm_table+t_contiguous,textpage);
 #endif
 
-  set_tm_maxpage(tm_table+t_relocatable,19);
+  set_tm_maxpage(tm_table+t_relocatable,1);
   nrbpage=0;
 
-#ifdef __linux__
-  /* Some versions of the Linux startup code are broken.
-     For these, the first call to sbrk() fails, but
-     subsequent calls are o.k.
-  */
-  if ( (long)sbrk(0) == -1 )
-    {
-      if ( (long)sbrk(0) == -1 )
-	{
-	  fputs("FATAL Linux sbrk() error\n", stderr);
-	  exit(1);
-	}
-      fputs("WARNING: Non-fatal Linux sbrk() error\n", stderr);
-    }
-#endif
-
   alloc_page(-(holepage + nrbpage));
   
   rb_start = rb_pointer = heap_end + PAGESIZE*holepage;
@@ -1147,21 +1171,12 @@ gcl_init_alloc(void) {
   tm_table[(int)t_relocatable].tm_sgc = 50;
 #endif
   
-#ifndef DONT_NEED_MALLOC	
-  
-  {
-    extern object malloc_list;
-    malloc_list = Cnil;
-    enter_mark_origin(&malloc_list);
-  }
-#endif	
-  
   gcl_alloc_initialized=1;
   
 }
 
-DEFUN_NEW("STATICP",object,fSstaticp,SI,1,1,NONE,OO,OO,OO,OO,(object x),"Tell if the string or vector is static") 
-{ RETURN1((inheap(x->ust.ust_self) ? sLt : sLnil));
+DEFUN_NEW("STATICP",object,fSstaticp,SI,1,1,NONE,OO,OO,OO,OO,(object x),"Tell if the string or vector is static") {
+  RETURN1((inheap(x->ust.ust_self) ? sLt : sLnil));
 }
 
 /* static void */
@@ -1239,15 +1254,16 @@ DEFUN_NEW("ALLOCATE-GROWTH",object,fSallocate_growth,SI,5,5,NONE,OO,II,II,OO,
 
 
 
-DEFUN_NEW("ALLOCATE-CONTIGUOUS-PAGES",object,fSallocate_contiguous_pages,SI,1,2,NONE,OI,OO,OO,OO,(fixnum npages,...),"") {
+DEFUN_NEW("ALLOCATE-CONTIGUOUS-PAGES",object,fSallocate_contiguous_pages,SI,1,2,NONE,OO,OO,OO,OO,(object onpages,...),"") {
 
   int nargs=VFUN_NARGS;
   object really_do;
   va_list ap;
+  fixnum npages=fixint(onpages);
   
   really_do=Cnil;
   if (nargs>=2) {
-    va_start(ap,npages);
+    va_start(ap,onpages);
     really_do=va_arg(ap,object);
     va_end(ap);
   }
@@ -1281,15 +1297,16 @@ DEFUN_NEW("MAXIMUM-CONTIGUOUS-PAGES",object,fSmaximum_contiguous_pages,SI,0,0,NO
 }
 
 
-DEFUN_NEW("ALLOCATE-RELOCATABLE-PAGES",object,fSallocate_relocatable_pages,SI,1,2,NONE,OI,OO,OO,OO,(fixnum npages,...),"") {
+DEFUN_NEW("ALLOCATE-RELOCATABLE-PAGES",object,fSallocate_relocatable_pages,SI,1,2,NONE,OO,OO,OO,OO,(object onpages,...),"") {
 
   int nargs=VFUN_NARGS;
   object really_do;
   va_list ap;
+  fixnum npages=fixint(onpages);
   
   really_do=Cnil;
   if (nargs>=2) {
-    va_start(ap,npages);
+    va_start(ap,onpages);
     really_do=va_arg(ap,object);
     va_end(ap);
   }
@@ -1307,17 +1324,18 @@ DEFUN_NEW("ALLOCATE-RELOCATABLE-PAGES",object,fSallocate_relocatable_pages,SI,1,
 
 }
 
-DEFUN_NEW("ALLOCATE",object,fSallocate,SI,2,3,NONE,OO,IO,OO,OO,(object type,fixnum npages,...),"") {
+DEFUN_NEW("ALLOCATE",object,fSallocate,SI,2,3,NONE,OO,OO,OO,OO,(object type,object onpages,...),"") {
 
   int nargs=VFUN_NARGS;
   object really_do;
   va_list ap;
   struct typemanager *tm;
+  fixnum npages=fixint(onpages);
   int t;
   
   really_do=Cnil;
   if (nargs>=3) {
-    va_start(ap,npages);
+    va_start(ap,onpages);
     really_do=va_arg(ap,object);
     va_end(ap);
   }
@@ -1325,9 +1343,9 @@ DEFUN_NEW("ALLOCATE",object,fSallocate,SI,2,3,NONE,OO,IO,OO,OO,(object type,fixn
   CHECK_ARG_RANGE(2,3);
   t= t_from_type(type);
   if (t == t_contiguous) 
-    RETURN1(FUNCALL(2,FFN(fSallocate_contiguous_pages)(npages,really_do)));
+    RETURN1(FUNCALL(2,FFN(fSallocate_contiguous_pages)(make_fixnum(npages),really_do)));
   else if (t==t_relocatable) 
-    RETURN1(FUNCALL(2,FFN(fSallocate_relocatable_pages)(npages,really_do)));
+    RETURN1(FUNCALL(2,FFN(fSallocate_relocatable_pages)(make_fixnum(npages),really_do)));
 
 
   if  (npages <= 0)
@@ -1358,8 +1376,6 @@ DEFUN_NEW("GET-HOLE-SIZE",object,fSget_hole_size,SI,0,0,NONE,OO,OO,OO,OO,(void),
 
 static unsigned long start,end,gprof_on;
 static void *initial_monstartup_pointer;
-bool capture_gprof_array=1;
-object gprof_array=Cnil;
 
 void
 gprof_cleanup(void) {
@@ -1368,7 +1384,6 @@ gprof_cleanup(void) {
 
   if (initial_monstartup_pointer) {
     _mcleanup();
-    gprof_array=Cnil;
     gprof_on=0;
   }
 
@@ -1393,20 +1408,26 @@ gprof_cleanup(void) {
 
 }
     
- 
+static inline int
+my_monstartup(unsigned long start,unsigned long end) {
 
-DEFUN_NEW("GPROF-START",object,fSgprof_start,SI
-       ,0,0,NONE,OO,OO,OO,OO,(void),"")
-{
   extern void monstartup(unsigned long,unsigned long);
+
+  monstartup(start,end);
+
+  return 0;
+
+}
+
+DEFUN_NEW("GPROF-START",object,fSgprof_start,SI,0,0,NONE,OO,OO,OO,OO,(void),"") {
+
   extern void *GCL_GPROF_START;
   static int n;
 
   if (!gprof_on) {
     start=start ? start : textstart;
     end=end ? end : textend;
-    capture_gprof_array=1;
-    monstartup(start,end);
+    writable_malloc_wrap(my_monstartup,int,start,end);
     gprof_on=1;
     if (!n && atexit(gprof_cleanup)) {
       FEerror("Cannot setup gprof_cleanup on exit", 0);
@@ -1445,7 +1466,6 @@ DEFUN_NEW("GPROF-QUIT",object,fSgprof_quit,SI
   if (chdir(P_tmpdir))
     FEerror("Cannot change directory to tmpdir", 0);
   _mcleanup();
-  gprof_array=Cnil;
   if (snprintf(b1,sizeof(b1),"gprof %s",kcl_self)<=0)
     FEerror("Cannot write gprof command line", 0);
   if (!(pp=popen(b1,"r")))
@@ -1465,19 +1485,19 @@ DEFUN_NEW("GPROF-QUIT",object,fSgprof_quit,SI
 
 #endif
 
-DEFUN_NEW("SET-STARTING-HOLE-DIVISOR",fixnum,fSset_starting_hole_divisor,SI,1,1,NONE,II,OO,OO,OO,(fixnum div),"") {
+DEFUN_NEW("SET-STARTING-HOLE-DIVISOR",object,fSset_starting_hole_divisor,SI,1,1,NONE,II,OO,OO,OO,(fixnum div),"") {
   if (div>0 && div <100)
     starting_hole_div=div;
-  return starting_hole_div;
+  return (object)starting_hole_div;
 }
   
-DEFUN_NEW("SET-STARTING-RELBLOCK-HEAP-MULTIPLE",fixnum,fSset_starting_relb_heap_multiple,SI,1,1,NONE,II,OO,OO,OO,(fixnum mult),"") {
+DEFUN_NEW("SET-STARTING-RELBLOCK-HEAP-MULTIPLE",object,fSset_starting_relb_heap_multiple,SI,1,1,NONE,II,OO,OO,OO,(fixnum mult),"") {
   if (mult>=0)
     starting_relb_heap_mult=mult;
-  return starting_relb_heap_mult;
+  return (object)starting_relb_heap_mult;
 }
   
-DEFUN_NEW("SET-HOLE-SIZE",object,fSset_hole_size,SI,1,2,NONE,OI,IO,OO,OO,(fixnum npages,...),"") {
+DEFUNM_NEW("SET-HOLE-SIZE",object,fSset_hole_size,SI,1,2,NONE,OO,OO,OO,OO,(object onpages,...),"") {
 
   printf("This function is obsolete -- use SET-STARTING-HOLE-DIVISOR instead\n");
 
@@ -1488,9 +1508,11 @@ DEFUN_NEW("SET-HOLE-SIZE",object,fSset_hole_size,SI,1,2,NONE,OI,IO,OO,OO,(fixnum
 
 void
 gcl_init_alloc_function(void) {
+
+  enter_mark_origin(&malloc_list);
+  
 }
 
-object malloc_list;
 
 #ifndef DONT_NEED_MALLOC
 
@@ -1572,109 +1594,83 @@ static char *baby_malloc(n)
 /*  } */
 /*  #endif */
 
+bool writable_malloc=0;
+
 void *
 malloc(size_t size) {
 
-  static int in_malloc;
+  static bool in_malloc;
 
   if (in_malloc)
     return NULL;
   in_malloc=1;
 
-  if (!GBC_enable) {
-
-#ifdef BABY_MALLOC_SIZE
-    in_malloc=0;
-    return baby_malloc(size);
-#else	
-
-    if (!initflag)
-      gcl_init_alloc();
-#ifdef RECREATE_HEAP
-    else RECREATE_HEAP
-#endif
-
-#endif	
-
-  }      
-
+  if (!gcl_alloc_initialized)
+    gcl_init_alloc(&size);
+  
   CHECK_INTERRUPT;
-
-  malloc_list = make_cons(Cnil, malloc_list);
-  
-  malloc_list->c.c_car = alloc_simple_string(size);
   
+  malloc_list = make_cons(alloc_simple_string(size), malloc_list);
   malloc_list->c.c_car->st.st_self = alloc_contblock(size);
+  malloc_list->c.c_car->st.st_adjustable=writable_malloc;
   
   /* FIXME: this is just to handle clean freeing of the
      monstartup memory allocated automatically on raw image
      startup.  In saved images, monstartup memory is only
      allocated with gprof-start. 20040804 CM*/
 #ifdef GCL_GPROF
-  if (!initflag && size>(textend-textstart) && !initial_monstartup_pointer) 
+  if (raw_image && size>(textend-textstart) && !initial_monstartup_pointer) {
+    massert(!atexit(gprof_cleanup));
     initial_monstartup_pointer=malloc_list->c.c_car->st.st_self;
-  if (gprof_array==Cnil && capture_gprof_array && size>(textend-textstart)) {
-    gprof_array=malloc_list->c.c_car;
-    capture_gprof_array=0;
   }
 #endif
   
   in_malloc=0;
   return(malloc_list->c.c_car->st.st_self);
-
+  
 }
 
 
 void
-free(void *ptr)
-#ifndef NO_VOID_STAR
-      
-#else
-      
-#endif  
-      
-{
+free(void *ptr) {
+
   object *p,pp;
-	if (ptr == 0)
-	  return;
-#ifdef BABY_MALLOC_SIZE
-	if ((void *)ptr < (void *) &baby_malloc_data[sizeof(baby_malloc_data)])
-	  return;
-#endif	
-	for (p = &malloc_list,pp=*p; pp && !endp(pp);  p = &((pp)->c.c_cdr),pp=Scdr(pp))
-		if ((pp)->c.c_car->st.st_self == ptr) {
-/* SGC contblock pages: Its possible this is on an old page CM 20030827 */
+  
+  if (ptr == 0)
+    return;
+  
+  for (p = &malloc_list,pp=*p; pp && !endp(pp);  p = &((pp)->c.c_cdr),pp=pp->c.c_cdr)
+    if ((pp)->c.c_car->st.st_self == ptr) {
+      /* SGC contblock pages: Its possible this is on an old page CM 20030827 */
 #ifdef SGC
- 			insert_maybe_sgc_contblock((pp)->c.c_car->st.st_self,
-						   (pp)->c.c_car->st.st_dim);
+      insert_maybe_sgc_contblock((pp)->c.c_car->st.st_self,(pp)->c.c_car->st.st_dim);
 #else
- 			insert_contblock((pp)->c.c_car->st.st_self,
-					 (pp)->c.c_car->st.st_dim);
+      insert_contblock((pp)->c.c_car->st.st_self,(pp)->c.c_car->st.st_dim);
 #endif
-			(pp)->c.c_car->st.st_self = NULL;
-			*p = Scdr(pp);
+      (pp)->c.c_car->st.st_self = NULL;
+      *p = pp->c.c_cdr;
 #ifdef GCL_GPROF
-			if (initial_monstartup_pointer==ptr) {
-			  initial_monstartup_pointer=NULL;
-			  if (core_end-heap_end>=sizeof(ptr))
-			    *(void **)heap_end=ptr;
-			}
+      if (initial_monstartup_pointer==ptr) {
+	initial_monstartup_pointer=NULL;
+	if (core_end-heap_end>=sizeof(ptr))
+	  *(void **)heap_end=ptr;
+      }
 #endif
-			return ;
-		}
+      return;
+    }
 #ifdef NOFREE_ERR
-	return ;
+  return;
 #else	
-	if (raw_image==FALSE || core_end-heap_end<sizeof(ptr) || ptr!=*(void **)heap_end) {
-	  static void *old_ptr;
-	  if (old_ptr==ptr) return;
-	  old_ptr=ptr;
-	  FEerror("free(3) error.",0);
-	}
-	return;
+  if (core_end-heap_end<sizeof(ptr) || ptr!=*(void **)heap_end) {
+    static void *old_ptr;
+    if (old_ptr==ptr) return;
+    old_ptr=ptr;
+    FEerror("free(3) error.",0);
+  }
+  return;
 #endif	
 }
-
+ 
 void *
 realloc(void *ptr, size_t size) {
 
