@@ -63,9 +63,10 @@ extern short aet_sizes[];
 #define N_FIXNUM_ARGS 6
 
 DEFUNO_NEW("AREF", object, fLaref, LISP, 1, ARRAY_RANK_LIMIT,
-       NONE, OO, II, II, II,void,Laref,(object x,fixnum i, ...),"")
+       NONE, OO, OO, OO, OO,void,Laref,(object x,object oi, ...),"")
 { int n = VFUN_NARGS;
   int i1;
+  fixnum i=n>1 ? fix(oi) : 0;
   va_list ap;
   if (type_of(x) == t_array)
     {int m ;
@@ -75,7 +76,7 @@ DEFUNO_NEW("AREF", object, fLaref, LISP, 1, ARRAY_RANK_LIMIT,
        FEerror(" ~a has wrong rank",1,x);
      if (rank == 1) return fLrow_major_aref(x,i);
      if (rank == 0) return fLrow_major_aref(x,0);
-     va_start(ap,i);
+     va_start(ap,oi);
      m = 0;
      k = i;
      /* index into 1 dimensional array */
@@ -90,7 +91,7 @@ DEFUNO_NEW("AREF", object, fLaref, LISP, 1, ARRAY_RANK_LIMIT,
 	 if (m <= rank)
 	   { i1 = i1 * x->a.a_dims[m];
 	     if (m < N_FIXNUM_ARGS)
-	       { k = va_arg(ap,fixnum);}
+	       { k = fixint(va_arg(ap,object));}
 	     else {object x = va_arg(ap,object);
 		   check_type(x,t_fixnum);
 		   k = Mfix(x);}
@@ -348,12 +349,13 @@ DEFUNO_NEW("SVSET", object, fSsvset, SI, 3, 3, NONE, OO, IO, OO,
 */ 
 
 
-DEFUN_NEW("MAKE-VECTOR1",object,fSmake_vector1,SI,3,8,NONE,OI,
-      IO,OO,OO,(fixnum n,fixnum elt_type,object staticp,...),"")
+DEFUN_NEW("MAKE-VECTOR1",object,fSmake_vector1,SI,3,8,NONE,OO,
+      OO,OO,OO,(object on,object oelt_type,object staticp,...),"")
   
 { 
     int displaced_index_offset=0;
     int Inargs = VFUN_NARGS - 3;
+    fixnum n=fixint(on),elt_type=fixint(oelt_type);
     va_list Iap;object fillp;object initial_element;object displaced_to;object V9;
     Inargs = VFUN_NARGS - 3 ;
     { object x;
@@ -441,7 +443,7 @@ DEFUN_NEW("MAKE-VECTOR1",object,fSmake_vector1,SI,3,8,NONE,OI,
 object 
 fSmake_vector1_1(fixnum n,fixnum elt_type,object staticp) {
   VFUN_NARGS=3;
-  return FFN(fSmake_vector1)(n,elt_type,staticp);
+  return FFN(fSmake_vector1)(make_fixnum(n),make_fixnum(elt_type),staticp);
 }
 
 
@@ -506,8 +508,8 @@ DEFUNO_NEW("MAKE-VECTOR",object,fSmake_vector,SI,7,8,NONE,
   /* 8 args */
 
   VFUN_NARGS = 8;
-  x = FFN(fSmake_vector1)(Mfix(x1),  /* n */
-		     fix(fSget_aelttype(x0)), /*aelt type */
+  x = FFN(fSmake_vector1)(x1,  /* n */
+		     fSget_aelttype(x0), /*aelt type */
 		     x6, /* staticp */
 		     x3, /* fillp */ 
 		     initial_elt, /* initial element */
@@ -548,6 +550,8 @@ DEFUN_NEW("MAKE-ARRAY1",object,fSmake_array1,SI,6,6,
       { x->a.a_dims[i] = FIX_CHECK(Mcar(v));
 	if (x->a.a_dims[i] < 0)
 	  { FEerror("Dimension must be non negative",0);}
+	if (dim && x->a.a_dims[i]>((1UL<<(sizeof(dim)*8-1))-1)/dim)
+	  FEerror("Total dimension overflow on dimensions ~s",1,dimensions);
 	dim *= x->a.a_dims[i++];
 	v = Mcdr(v);}
     x->a.a_dim = dim;
@@ -874,12 +878,13 @@ gset(void *p1, void *val, int n, int typ)
    */
 
 DEFUN_NEW("COPY-ARRAY-PORTION",object,fScopy_array_portion,SI,4,
-      5,NONE,OO,OI,IO,OO,(object x,object y,fixnum i1,fixnum i2,object n1o),
+      5,NONE,OO,OO,OO,OO,(object x,object y,object oi1,object oi2,object n1o),
    "Copy elements from X to Y starting at x[i1] to x[i2] and doing N1 \
 elements if N1 is supplied otherwise, doing the length of X - I1 \
 elements.  If the types of the arrays are not the same, this has \
 implementation dependent results.")
-{ enum aelttype typ1=Iarray_element_type(x);
+{ fixnum i1=fix(oi1),i2=fix(oi2);
+  enum aelttype typ1=Iarray_element_type(x);
   enum aelttype typ2=Iarray_element_type(y);
   int n1=fix(n1o),nc;
   if (VFUN_NARGS==4)
@@ -1158,10 +1163,9 @@ DEFUNO_NEW("REPLACE-ARRAY",object,fSreplace_array,SI,2,2,NONE,
   return old;
 }
 
-DEFUN_NEW("ARRAY-TOTAL-SIZE",object,fLarray_total_size,LISP,1,1,
-       NONE,OO,OO,OO,OO,(object x),"")
+DEFUN_NEW("ARRAY-TOTAL-SIZE",object,fLarray_total_size,LISP,1,1,NONE,IO,OO,OO,OO,(object x),"")
 { x = IisArray(x);
-  return make_fixnum(x->a.a_dim);
+  return (object)(fixnum)x->a.a_dim;
 }
 
 
