@@ -55,38 +55,7 @@ warn_avma()
 object sSAbreak_pointsA;
 object sSAbreak_stepA;
 
-
-/* This is a temporary workaround.  m68k cannot find the result 
-   of a function returning long when invoked via a function pointer
-   declared as a function returning a pointer, in this case, an 
-   object.  A proper fix will require rewriting sections of the lisp
-   compiler to separate the calling procedures for functions returning
-   an object from functions returning a long.  CM  20020801 */
-/*  #if defined(__mc68020__) */
-/*  #define LCAST(a) (object)(*(long(*)())a) */
-/*  #else */
-#define LCAST(a) (*a)
-/*  #endif */
-
-#define SET_TO_APPLY(res,f,n,x) \
- switch(n) {\
- case 0:   res=LCAST(f)(); break;\
-  case 1:  res=LCAST(f)(x[0]); break; \
-  case 2:  res=LCAST(f)(x[0],x[1]);break; \
-  case 3:  res=LCAST(f)(x[0],x[1],x[2]);break; \
-  case 4:  res=LCAST(f)(x[0],x[1],x[2],x[3]);break; \
-  case 5:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4]);break; \
-  case 6:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5]);  break;\
-  case 7:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5], x[6]); break;\
-  case 8:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5], x[6],x[7]); break;\
-  case 9:  res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5],x[6],x[7],x[8]);break;\
-  case 10: res=LCAST(f)(x[0],x[1],x[2],x[3],x[4],x[5],x[6],x[7],x[8],x[9]);break;\
-   default: res=c_apply_n(*f,n,x); break;}
-
-/*
-#undef SET_TO_APPLY
-#define SET_TO_APPLY(res,f,n,x)  res=c_apply_n(f,n,x);
-*/
+#include "apply_n.h"
 
 /* for t_sfun,t_gfun with args on vs stack */
 
@@ -114,7 +83,7 @@ quick_call_sfun(object fun)
      for (j=0; j<n ; j++)
        {enum ftype typ=SFUN_NEXT_TYPE(i);
 	x[j]=COERCE_ARG(vs_base[j],typ);}}
-  SET_TO_APPLY(res,fun->sfn.sfn_self,n,x);
+  res=c_apply_n_fun(fun,n,x);
   base[0]=
     (restype==f_object ?  res :
      restype==f_fixnum ? make_fixnum((long)res)
@@ -131,7 +100,7 @@ call_sfun_no_check(object fun)
   int n;
   object *base=vs_base;
   n=vs_top - base;
-  SET_TO_APPLY(base[0],fun->sfn.sfn_self,n,base);
+  base[0]=c_apply_n_fun(fun,n,base);
   vs_top=(vs_base=base)+1;
   CHECK_AVMA;
   return;
@@ -147,7 +116,7 @@ call_vfun(object fun)
   if (n > fun->vfn.vfn_maxargs)
     {FEtoo_many_arguments(base,vs_top); return;}
   VFUN_NARGS = n;
-  SET_TO_APPLY(base[0],fun->sfn.sfn_self,n,base);
+  base[0]=c_apply_n_fun(fun,n,base);
   vs_top=(vs_base=base)+1;
   CHECK_AVMA;
   return;
@@ -202,26 +171,13 @@ funcall(object fun)
 	     while (--n> 0 ) b[n] = fcall.values[n];
 	     b[0] = res;
 	     return;}		
-      case t_cclosure:
 
-	{
-		object *top, *base, l;
+        case t_cclosure:
+	
+	  MMccall(fun);
+	  CHECK_AVMA;
+	  return;
 
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			CHECK_AVMA; return;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		CHECK_AVMA; return;
-	}
-		
 	case t_symbol:
 	     {object x = fun->s.s_gfdef;
 	      if (x) { fun = x; goto TOP;}
@@ -360,23 +316,8 @@ funcall_no_event(object fun)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			(*fun->cc.cc_self)(fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		(*fun->cc.cc_self)(top);
-		break;
-	}
+	  (*fun->cc.cc_self)(fun);
+	  break;
 
 	case t_sfun:
 /*  		call_sfun_no_check(fun); return; */
@@ -408,26 +349,11 @@ lispcall(object *funp, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
+	  MMccall(fun);
+	  break;
 
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		break;
-	}
-
-	      default:
-		funcall(fun);
+	default:
+	  funcall(fun);
 
 	}
   CHECK_AVMA;
@@ -450,27 +376,12 @@ lispcall_no_event(object *funp, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			(*fun->cc.cc_self)(fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		(*fun->cc.cc_self)(top);
-		break;
-	}
 
+	  (*fun->cc.cc_self)(fun);
+	  break;
 
 	default:
-		funcall(fun);
+	  funcall(fun);
 
 	}
 	 CHECK_AVMA;
@@ -493,23 +404,8 @@ symlispcall(object sym, object *base, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		break;
-	}
+	  MMccall(fun);
+	  break;
 
 	default:
 		funcall(fun);
@@ -534,26 +430,11 @@ symlispcall_no_event(object sym, object *base, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			(*fun->cc.cc_self)(fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		(*fun->cc.cc_self)(top);
-		break;
-	}
+	  (*fun->cc.cc_self)(fun);
+	  break;
 
 	default:
-		funcall(fun);
+	  funcall(fun);
 
 	}
 	CHECK_AVMA;
@@ -577,26 +458,12 @@ simple_lispcall(object *funp, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		break;
-	}
+	  MMccall(fun);
+	  break;
 
 	default:
-		funcall(fun);
+	  funcall(fun);
+
 	}
 	vs_top = sup;
 	CHECK_AVMA;
@@ -667,26 +534,11 @@ simple_symlispcall(object sym, object *base, int narg)
 		break;
 
 	case t_cclosure:
-	{
-		object *top, *base, l;
-
-		if (fun->cc.cc_turbo != NULL) {
-			MMccall(fun, fun->cc.cc_turbo);
-			break;
-		}
-		top = vs_top;
-		base = vs_base;
-		for (l = fun->cc.cc_env;  !endp(l);  l = l->c.c_cdr)
-			vs_push(l);
-		vs_base = vs_top;
-		while (base < top)
-			vs_push(*base++);
-		MMccall(fun, top);
-		break;
-	}
+	  MMccall(fun);
+	  break;
 
 	default:
-		funcall(fun);
+	  funcall(fun);
 
 	}
 	vs_top = sup;
@@ -1138,7 +990,7 @@ call_applyhook(object fun)
 }
 
 
-DEFUNO_NEW("FUNCALL",object,fLfuncall,LISP
+DEFUNOM_NEW("FUNCALL",object,fLfuncall,LISP
        ,1,MAX_ARGS,NONE,OO,OO,OO,OO,void,Lfuncall,(object fun,...),"")
 { va_list ap;
   object *new;
@@ -1151,10 +1003,10 @@ DEFUNO_NEW("FUNCALL",object,fLfuncall,LISP
 }
 
 
-DEFUNO_NEW("APPLY",object,fLapply,LISP
+DEFUNOM_NEW("APPLY",object,fLapply,LISP
        ,2,MAX_ARGS,NONE,OO,OO,OO,OO,void,Lapply,(object fun,...),"")
 {	int m,n=VFUN_NARGS;
-	object list;
+	object l;
 	object buf[MAX_ARGS];
 	object *base=buf;
 	va_list ap;
@@ -1164,18 +1016,18 @@ DEFUNO_NEW("APPLY",object,fLapply,LISP
 	  {*base++ = va_arg(ap,object);
 	 }
 	m = n-2;
-	list = va_arg(ap,object);
+	l = va_arg(ap,object);
 	va_end(ap);
-	while (!endp(list))
+	while (!endp(l))
 	  { if (m >= MAX_ARGS) FEerror(" Lisps arglist maximum surpassed",0);
-	    *base++ = Mcar(list);
-	    list = Mcdr(list);
+	    *base++ = Mcar(l);
+	    l = Mcdr(l);
 	    m++;}
 	return IapplyVector(fun,m,buf);
       }
 	
 
-DEFUNO_NEW("EVAL",object,fLeval,LISP
+DEFUNOM_NEW("EVAL",object,fLeval,LISP
        ,1,1,NONE,OO,OO,OO,OO,void,Leval,(object x0),"")
 {
 	object *lex = lex_env;
@@ -1390,8 +1242,9 @@ int i=fcall.argd;
   	do{object fun=fcall.fun;
 		object *top, *base, l;
 
+		massert(fun->cc.cc_turbo);
 		if (fun->cc.cc_turbo != NULL) {
-			(*fun->cc.cc_self)(fun->cc.cc_turbo);
+			(*fun->cc.cc_self)(fun);
 			break;
 		}
 		top = vs_top;
@@ -1455,7 +1308,7 @@ fcalln_vfun(object first,va_list vl)
 {object *new,res;
  DEBUG_AVMA
  COERCE_VA_LIST_NEW(new,first,vl,fcall.argd);
- res = c_apply_n(fcall.fun->vfn.vfn_self,fcall.argd,new);
+ res = c_apply_n_fun(fcall.fun,fcall.argd,new);
  CHECK_AVMA;
  return res;
 }
