@@ -357,6 +357,19 @@ main(int argc, char **argv, char **envp) {
 
   gcl_init_alloc(&argv);
 
+#ifdef GET_FULL_PATH_SELF
+  GET_FULL_PATH_SELF(kcl_self);
+#else
+  kcl_self = argv[0];
+#endif
+#ifdef __MINGW32__
+  {
+    char *s=kcl_self;
+    for (;*s;s++) if (*s=='\\') *s='/';
+  }
+#endif	
+  *argv=kcl_self;
+  
 #ifdef CAN_UNRANDOMIZE_SBRK
 #include <stdio.h>
 #include <stdlib.h>
@@ -381,19 +394,6 @@ main(int argc, char **argv, char **envp) {
   ARGV = argv;
   ENVP = envp;
   
-#ifdef GET_FULL_PATH_SELF
-  GET_FULL_PATH_SELF(kcl_self);
-#else
-  kcl_self = argv[0];
-#endif
-#ifdef __MINGW32__
-  {
-    char *s=kcl_self;
-    for (;*s;s++) if (*s=='\\') *s='/';
-  }
-#endif	
-  *argv=kcl_self;
-  
   vs_top = vs_base = vs_org;
   ihs_top = ihs_org-1;
   bds_top = bds_org-1;
@@ -459,6 +459,8 @@ main(int argc, char **argv, char **envp) {
 /* catch certain signals */
 void install_segmentation_catcher(void)
 {
+  unblock_signals(SIGSEGV,SIGSEGV);
+  unblock_signals(SIGBUS,SIGBUS);
   (void) gcl_signal(SIGSEGV,segmentation_catcher);
   (void) gcl_signal(SIGBUS,segmentation_catcher);
 }
@@ -883,33 +885,38 @@ FFN(siLuser_init)(void) {
   vs_base[0] = Cnil;
 }
 
-static void
-FFN(siLaddress)(void) {
-  check_arg(1);
-  vs_base[0] = make_fixnum((long)vs_base[0]);
-}
+/* static void */
+/* FFN(siLaddress)(void) { */
+/*   check_arg(1); */
+/*   vs_base[0] = make_fixnum((long)vs_base[0]); */
+/* } */
 
-static void
-FFN(siLnani)(void) {
-  check_arg(1);
+DEFUN_NEW("NANI",object,fSnani,SI,1,1,NONE,OI,OO,OO,OO,(fixnum address),"") {
 
-  /*This is temporary, 2.6.x does not have 64bit fixnums on 64bit machines*/
-  switch (type_of(vs_base[0])) {
-  case t_fixnum:
-    vs_base[0]=(object)fix(vs_base[0]);
-    break;
-  case t_bignum:
-    if (mpz_fits_slong_p(MP(vs_base[0]))) {
-      MP_INT *u = MP(vs_base[0]);
-      vs_base[0]=(object)mpz_get_si(u);
-      break;
-    }
-  default:
-    FEerror("Cannot coerce ~s to an address",1,vs_base[0]);
-  }
+  RETURN1((object)address);
 
 }
 
+DEFUN_NEW("ADDRESS",object,fSaddress,SI,1,1,NONE,IO,OO,OO,OO,(object x),"") {
+
+  RETURN1(x);
+
+}
+
+/* static void */
+/* FFN(siLnani)(void) { */
+/*   check_arg(1); */
+
+/*   switch (type_of(vs_base[0])) { */
+/*   case t_fixnum: */
+/*     vs_base[0]=(object)fix(vs_base[0]); */
+/*     break; */
+/*   default: */
+/*     FEerror("Cannot coerce ~s to an address",1,vs_base[0]); */
+/*   } */
+
+/* } */
+
 static void
 FFN(siLinitialization_failure)(void) {
   check_arg(0);
@@ -994,8 +1001,8 @@ init_main(void) {
   make_si_function("RESET-STACK-LIMITS", siLreset_stack_limits);
   make_si_function("INIT-SYSTEM", siLinit_system);
   make_si_function("USER-INIT", siLuser_init);
-  make_si_function("ADDRESS", siLaddress);
-  make_si_function("NANI", siLnani);
+  /* make_si_function("ADDRESS", siLaddress); */
+  /* make_si_function("NANI", siLnani); */
   make_si_function("INITIALIZATION-FAILURE",
 		   siLinitialization_failure);
   
