@@ -45,6 +45,11 @@ extern void zero_divisor(void);
    * it is not zero, and
    * its exponent is non-zero.
 */
+
+#ifndef IEEEFLOAT
+#error this file needs IEEEFLOAT
+#endif
+
 int 
 gcl_isnormal_double(double d) {
 
@@ -53,12 +58,8 @@ gcl_isnormal_double(double d) {
   if (!ISFINITE(d) || !d)
     return 0;
 
-#ifdef IEEEFLOAT
   u.d = d;
   return (u.i[HIND] & 0x7ff00000) != 0;
-#else
-#error gcl_isnormal_double only implemented for IEEE
-#endif
 
 }
 
@@ -69,70 +70,11 @@ int gcl_isnormal_float(float f)
   if (!ISFINITE(f) || !f)
     return 0;
 
-#ifdef IEEEFLOAT
   u.f = f;
   return (u.i & 0x7f800000) != 0;
-#else
-#error gcl_isnormal_float only implemented for IEEE
-#endif
 
 }
 
-#ifdef VAX
-/*
-	radix = 2
-
-	SEEEEEEEEHHHHHHH	The redundant most significant fraction bit
-	HHHHHHHHHHHHHHHH	is not expressed.
-	LLLLLLLLLLLLLLLL
-	LLLLLLLLLLLLLLLL
-*/
-#endif
-#ifdef IBMRT
-
-
-
-
-
-
-
-
-#endif
-#ifdef IEEEFLOAT
-#ifdef NS32K
-
-
-
-
-
-
-
-#else
-/*
-	radix = 2
-
-	SEEEEEEEEEEEHHHHHHHHHHHHHHHHHHHH	The redundant most
-	LLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL	significant fraction bit
-						is not expressed.
-*/
-#endif
-#endif
-#ifdef MV
-
-
-
-
-
-
-#endif
-#ifdef S3000
-/*
-	radix = 16
-
-	SEEEEEEEHHHHHHHHHHHHHHHHHHHHHHHH
-	LLLLLLLLLLLLLLLLLLLLLLLLLLLLLLLL
-*/
-#endif
 static void
 integer_decode_double(double d, int *hp, int *lp, int *ep, int *sp)
 {
@@ -145,24 +87,9 @@ integer_decode_double(double d, int *hp, int *lp, int *ep, int *sp)
 		*sp = 1;
 		return;
 	}
-#ifdef NS32K
-
-
-#else
 	u.d=d;
 	h=u.i[HIND];
 	l=u.i[LIND];
-/* 	h = *((int *)(&d) + HIND); */
-/* 	l = *((int *)(&d) + LIND); */
-#endif
-#ifdef VAX
-	*ep = ((h >> 7) & 0xff) - 128 - 56;
-	h = ((h >> 15) & 0x1fffe) | (((h & 0x7f) | 0x80) << 17);
-	l = ((l >> 16) & 0xffff) | (l << 16);
-	/* is this right!!!! I don't believe it --wfs */
-	h = h >> 1;
-#endif
-#ifdef IEEEFLOAT
 	if (ISNORMAL(d)) {
 	  *ep = ((h & 0x7ff00000) >> 20) - 1022 - 53;
 	  h = ((h & 0x000fffff) | 0x00100000);
@@ -170,11 +97,6 @@ integer_decode_double(double d, int *hp, int *lp, int *ep, int *sp)
 	  *ep = ((h & 0x7fe00000) >> 20) - 1022 - 53 + 1;
 	  h = (h & 0x001fffff);
 	}
-#endif
-#ifdef S3000
-	*ep = ((h & 0x7f000000) >> 24) - 64 - 14;
-	h = (h & 0x00ffffff);
-#endif
 	if (32-BIG_RADIX)
 	  /* shift for making bignum */
 	  { h = h << (32-BIG_RADIX) ; 
@@ -186,45 +108,6 @@ integer_decode_double(double d, int *hp, int *lp, int *ep, int *sp)
 	*sp = (d > 0.0 ? 1 : -1);
 }
 
-#ifdef VAX
-/*
-	radix = 2
-
-	SEEEEEEEEMMMMMMM	The redundant most significant fraction bit
-	MMMMMMMMMMMMMMMM	is not expressed.
-*/
-#endif
-#ifdef IBMRT
-
-
-
-
-
-
-#endif
-#ifdef IEEEFLOAT
-/*
-	radix = 2
-
-	SEEEEEEEEMMMMMMMMMMMMMMMMMMMMMMM	The redundant most
-						significant fraction bit
-						is not expressed.
-*/
-#endif
-#ifdef MV
-
-
-
-
-
-#endif
-#ifdef S3000
-/*
-	radix = 16
-
-	SEEEEEEEMMMMMMMMMMMMMMMMMMMMMMMM
-*/
-#endif
 static void
 integer_decode_float(double d, int *mp, int *ep, int *sp)
 {
@@ -242,15 +125,6 @@ integer_decode_float(double d, int *mp, int *ep, int *sp)
 	u.f=f;
 	m=u.i;
 /* 	m = *(int *)(&f); */
-#ifdef VAX
-	*ep = ((m >> 7) & 0xff) - 128 - 24;
-	*mp = ((m >> 16) & 0xffff) | (((m & 0x7f) | 0x80) << 16);
-#endif
-#ifdef IBMRT
-
-
-#endif
-#ifdef IEEEFLOAT
 	if (ISNORMAL(f)) {
 	  *ep = ((m & 0x7f800000) >> 23) - 126 - 24;
 	  *mp = (m & 0x007fffff) | 0x00800000;
@@ -258,15 +132,6 @@ integer_decode_float(double d, int *mp, int *ep, int *sp)
 	  *ep = ((m & 0x7f000000) >> 23) - 126 - 24 + 1;
 	  *mp = m & 0x00ffffff;
 	}
-#endif
-#ifdef MV
-
-
-#endif
-#ifdef S3000
-	*ep = ((m & 0x7f000000) >> 24) - 64 - 6;
-	*mp = m & 0x00ffffff;
-#endif
 	*sp = (f > 0.0 ? 1 : -1);
 }
 
@@ -277,26 +142,8 @@ double_exponent(double d)
 
 	if (d == 0.0)
 		return(0);
-#ifdef VAX
-	return(((*(int *)(&d) >> 7) & 0xff) - 128);
-#endif
-#ifdef IBMRT
-
-#endif
-#ifdef IEEEFLOAT
-#ifdef NS32K
-
-#else
 	u.d=d;
 	return (((u.i[HIND] & 0x7ff00000) >> 20) - 1022);
-#endif
-#endif
-#ifdef MV
-
-#endif
-#ifdef S3000
-	return(((*(int *)(&d) & 0x7f000000) >> 24) - 64);
-#endif
 }
 
 static double
@@ -308,91 +155,41 @@ set_exponent(double d, int e)
 		return(0.0);
 	  
 	u.d=d;
-	u.i[HIND]
-#ifdef VAX
-	= *(int *)(&d) & 0xffff807f | ((e + 128) << 7) & 0x7f80;
-#endif
-#ifdef IBMRT
-
-#endif
-#ifdef IEEEFLOAT
-#ifdef NS32K
-
-#else
-	= (u.i[HIND] & 0x800fffff) | (((e + 1022) << 20) & 0x7ff00000);
-#endif
-#endif
-#ifdef MV
-
-#endif
-#ifdef S3000
-	= *(int *)(&d) & 0x80ffffff | ((e + 64) << 24) & 0x7f000000;
-#endif
+	u.i[HIND]= (u.i[HIND] & 0x800fffff) | (((e + 1022) << 20) & 0x7ff00000);
 	return(u.d);
 }
 
 
 object
-double_to_integer(double d)
-{
-	int h, l, e, s;
-	object x;
-	vs_mark;
+double_to_integer(double d) {
 
-	if (d == 0.0)
-		return(small_fixnum(0));
-	integer_decode_double(d, &h, &l, &e, &s);
-#ifdef VAX
-	if (e <= -BIG_RADIX) {
-		h >>= (-e) - BIG_RADIX;
-#endif
-#ifdef IBMRT
+  int h, l, e, s;
+  object x;
+  vs_mark;
+  
+  if (d == 0.0)
+    return(small_fixnum(0));
+  integer_decode_double(d, &h, &l, &e, &s);
 
-
-#endif
-#ifdef IEEEFLOAT
-	if (e <= -BIG_RADIX) {
-		e = (-e) - BIG_RADIX;
-		if (e >= BIG_RADIX)
-			return(small_fixnum(0));
-		h >>= e;
-#endif
-#ifdef MV
-
-
-#endif
-#ifdef S3000
-	if (e <= -8) {
-		h >>= 4*(-e) - BIG_RADIX;
-#endif
-		return(make_fixnum(s*h));
-	}
-	if (h != 0 || l<0)
-		x = bignum2(h, l);
-	else
-		x = make_fixnum(l);
-	vs_push(x);
-#ifdef VAX
-	x = shift_integer(x, e);
-#endif
-#ifdef IBMRT
-
-#endif
-#ifdef IEEEFLOAT
-	x = shift_integer(x, e);
-#endif
-#ifdef MV
-
-#endif
-#ifdef S3000
-	x = shift_integer(x, 4*e);
-#endif
-	if (s < 0) {
-		vs_push(x);
-		x = number_negate(x);
-	}
-	vs_reset;
-	return(x);
+  if (e <= -BIG_RADIX) {
+    e = (-e) - BIG_RADIX;
+    if (e >= BIG_RADIX)
+      return(small_fixnum(0));
+    h >>= e;
+    return(make_fixnum(s*h));
+  }
+  if (h != 0 || l<0)
+    x = bignum2(h, l);
+  else
+    x = make_fixnum(l);
+  vs_push(x);
+  x = integer_fix_shift(x, e);
+  if (s < 0) {
+    vs_push(x);
+    x = number_negate(x);
+  }
+  vs_reset;
+  return(x);
 }
 
 static object
@@ -480,343 +277,145 @@ LFD(Ldenominator)(void)
 		vs_base[0] = small_fixnum(1);
 }
 
-LFD(Lfloor)(void)
-{
-	object x, y, q, q1;
-	double d;
-	int n;
-	object one_minus(object x);
+inline void
+intdivrem(object x,object y,fixnum d,object *q,object *r) {
 
-	n = vs_top - vs_base;
-	if (n == 0)
-		too_few_arguments();
-	if (n > 1)
-		goto TWO_ARG;
-	x = vs_base[0];
-	switch (type_of(x)) {
+  enum type tx=type_of(x),ty=type_of(y);
+  object z,q2,q1;
 
-	case t_fixnum:
-	case t_bignum:
-		vs_push(small_fixnum(0));
-		return;
+  if (number_zerop(y)==TRUE)
+    zero_divisor();
 
-	case t_ratio:
-		q = x;
-		y = small_fixnum(1);
-		goto RATIO;
+  switch(tx) {
+  case t_fixnum:
+  case t_bignum:
+    switch (ty) {
+    case t_fixnum:
+    case t_bignum:
+      integer_quotient_remainder_1(x,y,q,r,d);
+      return;
+    case t_ratio:
+      z=integer_divide1(number_times(y->rat.rat_den,x),y->rat.rat_num,d);
+      if (q) *q=z;
+      if (r) *r=num_remainder(x,y,z);
+      return;
+    default:
+      break;
+    }
+    break;
+  case t_ratio:
+    switch (ty) {
+    case t_fixnum:
+    case t_bignum:
+      z=integer_divide1(x->rat.rat_num,number_times(x->rat.rat_den,y),d);
+      if (q) *q=z;
+      if (r) *r=num_remainder(x,y,z);
+      return;
+    case t_ratio:
+      z=integer_divide1(number_times(x->rat.rat_num,y->rat.rat_den),number_times(x->rat.rat_den,y->rat.rat_num),d);
+      if (q) *q=z;
+      if (r) *r=num_remainder(x,y,z);
+      return;
+    default:
+      break;
+    }
+    break;
+  default:
+    break;
+  }
 
-	case t_shortfloat:
-		d = (double)(sf(x));
-		q1 = double_to_integer(d);
-		d -= number_to_double(q1);
-		if (sf(x) < 0.0 && d != 0.0) {
-			vs_push(q1);
-			q1 = one_minus(q1);
-			d += 1.0;
-		}
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(make_shortfloat((shortfloat)d));
-		return;
-
-	case t_longfloat:
-		d = lf(x);
-		q1 = double_to_integer(d);
-		d -= number_to_double(q1);
-		if (lf(x) < 0.0 && d != 0.0) {
-			vs_push(q1);
-			q1 = one_minus(q1);
-			d += 1.0;
-		}
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(make_longfloat(d));
-		return;
-
-	default:
-		FEwrong_type_argument(TSor_rational_float, x);
-	}
-
-TWO_ARG:
-	if (n > 2)
-		too_many_arguments();
-	x = vs_base[0];
-	y = vs_base[1];
-        if ( number_zerop ( y ) == TRUE ) {
-            zero_divisor();
-        }
-	if ((type_of(x) == t_fixnum || type_of(x) == t_bignum) &&
-	    (type_of(y) == t_fixnum || type_of(y) == t_bignum)) {
-		vs_base = vs_top;
-		if (number_zerop(x)) {
-			vs_push(small_fixnum(0));
-			vs_push(small_fixnum(0));
-			return;
-		}
-		vs_push(Cnil);
-		vs_push(Cnil);
-		integer_quotient_remainder_1(x, y, &vs_base[0], &vs_base[1]);
-		if (number_minusp(x) ? number_plusp(y) : number_minusp(y)) {
-			if (number_zerop(vs_base[1]))
-				return;
-			vs_base[0] = one_minus(vs_base[0]);
-			vs_base[1] = number_plus(vs_base[1], y);
-		}
-		return;
-	}
-	check_type_or_rational_float(&vs_base[0]);
-	check_type_or_rational_float(&vs_base[1]);
-	q = number_divide(x, y);
-	vs_push(q);
-	switch (type_of(q)) {
-	case t_fixnum:
-	case t_bignum:
-		vs_base = vs_top;
-		vs_push(q);
-		vs_push(small_fixnum(0));
-		break;
-	
-	case t_ratio:
-	RATIO:
-		q1 = integer_divide1(q->rat.rat_num, q->rat.rat_den);
-		if (number_minusp(q)) {
-			vs_push(q1);
-			q1 = one_minus(q1);
-		} else
-			q1 = q1;
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(num_remainder(x, y, q1));
-		return;
-
-	case t_shortfloat:
-	case t_longfloat:
-		q1 = double_to_integer(number_to_double(q));
-		if (number_minusp(q) && number_compare(q, q1)) {
-			vs_push(q1);
-			q1 = one_minus(q1);
-		} else
-			q1 = q1;
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(num_remainder(x, y, q1));
-		return;
-	default:
-	  break;
-	}
+  q2=number_divide(x,y);
+  q1=double_to_integer(number_to_double(q2));
+  if (d && (d<0 ? number_minusp(q2) : number_plusp(q2)) && number_compare(q2, q1))
+    q1 = d<0 ? one_minus(q1) : one_plus(q1);
+  if (q) *q=q1;
+  if (r) *r=num_remainder(x,y,q1);
+  return;
+  
 }
 
-LFD(Lceiling)(void)
-{
-	object x, y, q, q1;
-	double d;
-	int n;
-	object one_plus(object x);
-
-	n = vs_top - vs_base;
-	if (n == 0)
-		too_few_arguments();
-	if (n > 1)
-		goto TWO_ARG;
-	x = vs_base[0];
-	switch (type_of(x)) {
-
-	case t_fixnum:
-	case t_bignum:
-		vs_push(small_fixnum(0));
-		return;
-
-	case t_ratio:
-		q = x;
-		y = small_fixnum(1);
-		goto RATIO;		
-
-	case t_shortfloat:
-		d = (double)(sf(x));
-		q1 = double_to_integer(d);
-		d -= number_to_double(q1);
-		if (sf(x) > 0.0 && d != 0.0) {
-			vs_push(q1);
-			q1 = one_plus(q1);
-			d -= 1.0;
-		}
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(make_shortfloat((shortfloat)d));
-		return;
-
-	case t_longfloat:
-		d = lf(x);
-		q1 = double_to_integer(d);
-		d -= number_to_double(q1);
-		if (lf(x) > 0.0 && d != 0.0) {
-			vs_push(q1);
-			q1 = one_plus(q1);
-			d -= 1.0;
-		}
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(make_longfloat(d));
-		return;
-
-	default:
-		FEwrong_type_argument(TSor_rational_float, x);
-	}
-
-TWO_ARG:
-	if (n > 2)
-		too_many_arguments();
-	x = vs_base[0];
-	y = vs_base[1];
-        if ( number_zerop ( y ) == TRUE ) {
-            zero_divisor();
-        }
-	if ((type_of(x) == t_fixnum || type_of(x) == t_bignum) &&
-	    (type_of(y) == t_fixnum || type_of(y) == t_bignum)) {
-		vs_base = vs_top;
-		if (number_zerop(x)) {
-			vs_push(small_fixnum(0));
-			vs_push(small_fixnum(0));
-			return;
-		}
-		vs_push(Cnil);
-		vs_push(Cnil);
-		integer_quotient_remainder_1(x, y, &vs_base[0], &vs_base[1]);
-		if (number_plusp(x) ? number_plusp(y) : number_minusp(y)) {
-			if (number_zerop(vs_base[1]))
-				return;
-			vs_base[0] = one_plus(vs_base[0]);
-			vs_base[1] = number_minus(vs_base[1], y);
-		}
-		return;
-	}
-	check_type_or_rational_float(&vs_base[0]);
-	check_type_or_rational_float(&vs_base[1]);
-	q = number_divide(x, y);
-	vs_push(q);
-	switch (type_of(q)) {
-	case t_fixnum:
-	case t_bignum:
-		vs_base = vs_top;
-		vs_push(q);
-		vs_push(small_fixnum(0));
-		break;
-	
-	case t_ratio:
-	RATIO:
-		q1 = integer_divide1(q->rat.rat_num, q->rat.rat_den);
-		if (number_plusp(q)) {
-			vs_push(q1);
-			q1 = one_plus(q1);
-		} else
-			q1 = q1;
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(num_remainder(x, y, q1));
-		return;
-
-	case t_shortfloat:
-	case t_longfloat:
-		q1 = double_to_integer(number_to_double(q));
-		if (number_plusp(q) && number_compare(q, q1)) {
-			vs_push(q1);
-			q1 = one_plus(q1);
-		} else
-			q1 = q1;
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(num_remainder(x, y, q1));
-		return;
-	default:
-	  break;
-	}
+object
+number_ldb(object x,object y) {
+  return ifuncall2(sLldb,x,y);
 }
 
-LFD(Ltruncate)(void)
-{
-	object x, y, q, q1;
-	int n;
+object
+number_ldbt(object x,object y) {
+  return ifuncall2(sLldb_test,x,y);
+}
 
-	n = vs_top - vs_base;
-	if (n == 0)
-		too_few_arguments();
-	if (n > 1)
-		goto TWO_ARG;
-	x = vs_base[0];
-	switch (type_of(x)) {
+object
+number_dpb(object x,object y,object z) {
+  return ifuncall3(sLdpb,x,y,z);
+}
 
-	case t_fixnum:
-	case t_bignum:
-		vs_push(small_fixnum(0));
-		return;
+object
+number_dpf(object x,object y,object z) {
+  return ifuncall3(sLdeposit_field,x,y,z);
+}
 
-	case t_ratio:
-		q1 = integer_divide1(x->rat.rat_num, x->rat.rat_den);
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(number_minus(x, q1));
-		return;
 
-	case t_shortfloat:
-		q1 = double_to_integer((double)(sf(x)));
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(number_minus(x, q1));
-		return;
+LFD(Lfloor)(void) {
 
-	case t_longfloat:
-		q1 = double_to_integer(lf(x));
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(number_minus(x, q1));
-		return;
+  object x, y;
+  int n = vs_top - vs_base;
 
-	default:
-		FEwrong_type_argument(TSor_rational_float, x);
-	}
+  if (n == 0)
+    too_few_arguments();
+  if (n > 2)
+    too_many_arguments();
 
-TWO_ARG:
-	if (n > 2)
-		too_many_arguments();
-	x = vs_base[0];
-	y = vs_base[1];
-        if ( number_zerop ( y ) == TRUE ) {
-            zero_divisor();
-        }
-	if ((type_of(x) == t_fixnum || type_of(x) == t_bignum) &&
-	    (type_of(y) == t_fixnum || type_of(y) == t_bignum)) {
-		integer_quotient_remainder_1(x, y, &vs_base[0], &vs_base[1]);
-		return;
-	}
-	check_type_or_rational_float(&vs_base[0]);
-	check_type_or_rational_float(&vs_base[1]);
-	q = number_divide(x, y);
-	vs_push(q);
-	switch (type_of(q)) {
-	case t_fixnum:
-	case t_bignum:
-		vs_base = vs_top;
-		vs_push(q);
-		vs_push(small_fixnum(0));
-		break;
-	
-	case t_ratio:
-		q1 = integer_divide1(q->rat.rat_num, q->rat.rat_den);
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(num_remainder(x, y, q1));
-		return;
+  x = vs_base[0];
+  y = n>1 ? vs_base[1] : small_fixnum(1);
+
+  intdivrem(x,y,-1,&x,&y);
+
+  vs_top=vs_base;
+  vs_push(x);
+  vs_push(y);
+
+}
+
+LFD(Lceiling)(void) {
+
+  object x, y;
+  int n = vs_top - vs_base;
+
+  if (n == 0)
+    too_few_arguments();
+  if (n > 2)
+    too_many_arguments();
+
+  x = vs_base[0];
+  y = n>1 ? vs_base[1] : small_fixnum(1);
+
+  intdivrem(x,y,1,&x,&y);
+
+  vs_top=vs_base;
+  vs_push(x);
+  vs_push(y);
+
+}
+
+LFD(Ltruncate)(void) {
+
+  object x, y;
+  int n = vs_top - vs_base;
+
+  if (n == 0)
+    too_few_arguments();
+  if (n > 2)
+    too_many_arguments();
+
+  x = vs_base[0];
+  y = n>1 ? vs_base[1] : small_fixnum(1);
+
+  intdivrem(x,y,0,&x,&y);
+
+  vs_top=vs_base;
+  vs_push(x);
+  vs_push(y);
 
-	case t_shortfloat:
-	case t_longfloat:
-		q1 = double_to_integer(number_to_double(q));
-		vs_base = vs_top;
-		vs_push(q1);
-		vs_push(num_remainder(x, y, q1));
-		return;
-	default:
-	  break;
-	}
 }
 
 LFD(Lround)(void)
@@ -911,7 +510,7 @@ TWO_ARG:
 	
 	case t_ratio:
 	RATIO:
-		q1 = integer_divide1(q->rat.rat_num, q->rat.rat_den);
+	  q1 = integer_divide1(q->rat.rat_num, q->rat.rat_den,0);/*FIXME*/
 		vs_push(q1);
 		r = number_minus(q, q1);
 		vs_push(r);
@@ -951,18 +550,18 @@ TWO_ARG:
 	}
 }
 
-LFD(Lmod)(void)
-{
-	check_arg(2);
-	Lfloor();
-	vs_base++;
-}
 
-LFD(Lrem)(void)
-{
-	check_arg(2);
-	Ltruncate();
-	vs_base++;
+LFD(Lmod)(void) {
+  check_arg(2);
+  intdivrem(vs_base[0],vs_base[1],-1,NULL,vs_base);
+  vs_top=vs_base+1;
+}
+  
+
+LFD(Lrem)(void) {
+  check_arg(2);
+  intdivrem(vs_base[0],vs_base[1],0,NULL,vs_base);
+  vs_top=vs_base+1;
 }
 
 
