@@ -338,8 +338,8 @@ DEFUNO_NEW("FUNCTIONP",object,fLfunctionp,LISP
 		    x0->s.s_mflag == FALSE)
 			x0 = Ct;
 		else
-			x0 = Cnil;
-	} else if (t == t_cons) {
+			x0 = Cnil; }
+	else if (t == t_cons) {
 		x = x0->c.c_car;
 		if (x == sLlambda || x == sLlambda_block ||
 		    x == sSlambda_block_expanded ||
