@@ -386,7 +386,11 @@ int	server;
 		FEerror("Connection Failed.",0);
 	}
 	pid = getpid();
+#ifdef __CYGWIN__
+	if(fcntl(sock, F_SETOWN, pid) < 0)
+#else
 	if(ioctl(sock, SIOCSPGRP, (char *)&pid) < 0 )
+#endif
 	{
 		FEerror("Could not set process group of socket.",0);
 	}
@@ -531,7 +535,7 @@ char **argv;
     FEerror("Cannot spawn process with given stream", 0);
   fdin = istream->sm.sm_int0;
   fdout = ostream->sm.sm_int0;
-  if (fork() == 0)
+  if (pfork() == 0)
     { /* the child --- replace standard in and out with descriptors given */
       close(0);
       massert(dup(fdin)>=0);
