@@ -110,7 +110,6 @@ void recreate_heap1()
       printf ("Failed to find path for executable.\n");
       exit (1);
     }
-
     recreate_heap (executable_path);
   }
   heap_state = HEAP_LOADED;
@@ -220,6 +219,7 @@ unexec (char *new_name, char *old_name, void *start_data, void *start_bss,
   char filename[MAX_PATH];
   unsigned long size;
   char *ptr;
+  extern void cygwin_conv_to_full_win32_path(char *,char *);
 
   fflush (stdin);
   /* copy_stdin = *stdin; */
@@ -970,20 +970,29 @@ allocate_heap (void)
      the region below the 256MB line for our malloc arena - 229MB is
      still a pretty decent arena to play in!  */
 
-  void *base = (void *)0x20000000,*ptr;/*FIXME, someday figure out how to let the heap start address default *//*(void *)0x10100000*/
+#if defined(__CYGWIN__)
+#define PROBE_BASE NULL
+#elif defined(__MINGW32__)
+#define PROBE_BASE (void *)0x20000000
+#else
+#error Need PROBE_BASE
+#endif
+
+  void *base = PROBE_BASE,*ptr;/*FIXME, someday figure out how to let the heap start address default *//*(void *)0x10100000*/
 
   reserved_heap_size=probe_heap_size(base,PAGESIZE,(1UL<<31),-1);
   ptr = VirtualAlloc ((void *) base,get_reserved_heap_size (),MEM_RESERVE,PAGE_NOACCESS);
-  printf("probe results: %lu at %p\n",reserved_heap_size,ptr);
+  /* printf("probe results: %lu at %p\n",reserved_heap_size,ptr); */
 
   DBEGIN = (DBEGIN_TY) ptr;
 
   return ptr;
+
 }
 
 /* Emulate Unix sbrk.  */
 void *
-sbrk (unsigned long increment)
+sbrk (ptrdiff_t increment)
 {
   void *result;
   long size = (long) increment;
@@ -1061,11 +1070,16 @@ sbrk (unsigned long increment)
   return result;
 }
 
+#ifdef __CYGWIN__
+/* Emulate Unix getpagesize.  */
+int getpagesize (void) { return 4096; }
+#endif
+
 /* Recreate the heap from the data that was dumped to the executable.
    EXECUTABLE_PATH tells us where to find the executable.  */
 void
-recreate_heap (char *executable_path)
-{
+recreate_heap (char *executable_path) {
+
   unsigned char *tmp;
 
   /* First reserve the upper part of our heap.  (We reserve first
@@ -1088,6 +1102,7 @@ recreate_heap (char *executable_path)
 
   /* Update system version information to match current system.  */
   cache_system_info ();
+
 }
 
 /* Round the heap up to the given alignment.  */
