@@ -30,6 +30,7 @@ gcl_init_system(object no_init)
   lsp_init("../lsp/gcl_arraylib.lsp");
   lsp_init("../lsp/gcl_assert.lsp");
   lsp_init("../lsp/gcl_defstruct.lsp");
+  lsp_init("../lsp/gcl_restart.lsp");
   lsp_init("../lsp/gcl_describe.lsp");
 #ifdef HAVE_JAPI_H
   lsp_init("../lsp/gcl_japi.lsp");
@@ -49,6 +50,7 @@ gcl_init_system(object no_init)
   lsp_init("../lsp/gcl_defpackage.lsp");
   lsp_init("../lsp/gcl_make_defpackage.lsp");
   lsp_init("../lsp/gcl_sharp.lsp");
+  lsp_init("../lsp/gcl_fpe.lsp");
 
   lsp_init("../cmpnew/gcl_cmpinline.lsp");
   lsp_init("../cmpnew/gcl_cmputil.lsp");
